@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -23,10 +23,13 @@
 #ifndef QGLVIEWER_CAMERA_H
 #define QGLVIEWER_CAMERA_H
 
-#include "manipulatedCameraFrame.h"
 #include "keyFrameInterpolator.h"
+class QGLViewer;
 
 namespace qglviewer {
+
+class ManipulatedCameraFrame;
+
 /*! \brief A perspective or orthographic camera.
   \class Camera camera.h QGLViewer/camera.h
 
@@ -49,7 +52,7 @@ namespace qglviewer {
 
   The default button binding are: QGLViewer::ROTATE (left), QGLViewer::ZOOM (middle) and
   QGLViewer::TRANSLATE (right). With this configuration, the Camera \e observes a scene and rotates
-  around its revolveAroundPoint(). You can switch between this mode and a fly mode using the
+  around its pivotPoint(). You can switch between this mode and a fly mode using the
   QGLViewer::CAMERA_MODE (see QGLViewer::toggleCameraMode()) keyboard shortcut (default is 'Space').
 
   <h3>Other functionalities</h3>
@@ -98,66 +101,25 @@ public:
 
 	See type() and setType(). This type mainly defines different Camera projection matrix (see
 	loadProjectionMatrix()). Many other methods (pointUnderPixel(), convertClickToLine(),
-	projectedCoordinatesOf(), pixelGLRatio()...) take this Type into account. */
+	projectedCoordinatesOf(), pixelGLRatio()...) are affected by this Type. */
 	enum Type { PERSPECTIVE, ORTHOGRAPHIC };
 
 	/*! @name Position and orientation */
 	//@{
 public:
-	/*! Returns the Camera position (the eye), defined in the world coordinate system.
-
-	Use setPosition() to set the Camera position. Other convenient methods are showEntireScene() or
-	fitSphere(). Actually returns \c frame()->position().
-
-	This position corresponds to the projection center of a Camera::PERSPECTIVE Camera. It is not
-	located in the image plane, which is at a zNear() distance ahead. */
-	Vec position() const { return frame()->position(); }
-
-	/*! Returns the normalized up vector of the Camera, defined in the world coordinate system.
-
-	Set using setUpVector() or setOrientation(). It is orthogonal to viewDirection() and to
-	rightVector().
-
-	It corresponds to the Y axis of the associated frame() (actually returns
-	frame()->inverseTransformOf(Vec(0.0, 1.0, 0.0)) ). */
-	Vec upVector() const
-	{
-		return frame()->inverseTransformOf(Vec(0.0, 1.0, 0.0));
-	}
-	/*! Returns the normalized view direction of the Camera, defined in the world coordinate system.
-
-	Change this value using setViewDirection(), lookAt() or setOrientation(). It is orthogonal to
-	upVector() and to rightVector().
-
-	This corresponds to the negative Z axis of the frame() ( frame()->inverseTransformOf(Vec(0.0,
-	0.0, -1.0)) ). */
-	Vec viewDirection() const { return frame()->inverseTransformOf(Vec(0.0, 0.0, -1.0)); }
-
-	/*! Returns the normalized right vector of the Camera, defined in the world coordinate system.
-
-	This vector lies in the Camera horizontal plane, directed along the X axis (orthogonal to
-	upVector() and to viewDirection()). Set using setUpVector(), lookAt() or setOrientation().
-
-	Simply returns frame()->inverseTransformOf(Vec(1.0, 0.0, 0.0)). */
-	Vec rightVector() const
-	{
-		return frame()->inverseTransformOf(Vec(1.0, 0.0, 0.0));
-	}
-
-	/*! Returns the Camera orientation, defined in the world coordinate system.
-
-	Actually returns \c frame()->orientation(). Use setOrientation(), setUpVector() or lookAt() to
-	set the Camera orientation. */
-	Quaternion orientation() const { return frame()->orientation(); }
+	Vec position() const;
+	Vec upVector() const;
+	Vec viewDirection() const;
+	Vec rightVector() const;
+	Quaternion orientation() const;
 
 	void setFromModelViewMatrix(const GLdouble* const modelViewMatrix);
-	void setFromProjectionMatrix(const float matrix[12]);
+	void setFromProjectionMatrix(const qreal matrix[12]);
 
 public Q_SLOTS:
-	/*! Sets the Camera position() (the eye), defined in the world coordinate system. */
-	void setPosition(const Vec& pos) { frame()->setPosition(pos); }
+	void setPosition(const Vec& pos);
 	void setOrientation(const Quaternion& q);
-	void setOrientation(float theta, float phi);
+	void setOrientation(qreal theta, qreal phi);
 	void setUpVector(const Vec& up, bool noMove=true);
 	void setViewDirection(const Vec& direction);
 	//@}
@@ -168,13 +130,13 @@ public Q_SLOTS:
 public Q_SLOTS:
 	void lookAt(const Vec& target);
 	void showEntireScene();
-	void fitSphere(const Vec& center, float radius);
+	void fitSphere(const Vec& center, qreal radius);
 	void fitBoundingBox(const Vec& min, const Vec& max);
 	void fitScreenRegion(const QRect& rectangle);
 	void centerScene();
 	void interpolateToZoomOnPixel(const QPoint& pixel);
 	void interpolateToFitScene();
-	void interpolateTo(const Frame& fr, float duration);
+	void interpolateTo(const Frame& fr, qreal duration);
 	//@}
 
 
@@ -188,7 +150,7 @@ public:
 	A Camera::PERSPECTIVE Camera uses a classical projection mainly defined by its fieldOfView().
 
 	With a Camera::ORTHOGRAPHIC type(), the fieldOfView() is meaningless and the width and height of
-	the Camera frustum are inferred from the distance to the revolveAroundPoint() using
+	the Camera frustum are inferred from the distance to the pivotPoint() using
 	getOrthoWidthHeight().
 
 	Both types use zNear() and zFar() (to define their clipping planes) and aspectRatio() (for
@@ -205,7 +167,7 @@ public:
 	ratio (see aspectRatio() and horizontalFieldOfView()).
 
 	Use setFOVToFitScene() to adapt the fieldOfView() to a given scene. */
-	float fieldOfView() const { return fieldOfView_; }
+	qreal fieldOfView() const { return fieldOfView_; }
 
 	/*! Returns the horizontal field of view of the Camera (in radians).
 
@@ -214,13 +176,13 @@ public:
 	\code
 	horizontalFieldOfView() = 2.0 * atan ( tan(fieldOfView()/2.0) * aspectRatio() ).
 	\endcode */
-	float horizontalFieldOfView() const { return 2.0 * atan ( tan(fieldOfView()/2.0) * aspectRatio() ); }
+	qreal horizontalFieldOfView() const { return 2.0 * atan ( tan(fieldOfView()/2.0) * aspectRatio() ); }
 
 	/*! Returns the Camera aspect ratio defined by screenWidth() / screenHeight().
 
 	When the Camera is attached to a QGLViewer, these values and hence the aspectRatio() are
 	automatically fitted to the viewer's window aspect ratio using setScreenWidthAndHeight(). */
-	float aspectRatio() const { return static_cast<float>(screenWidth_)/static_cast<float>(screenHeight_); }
+	qreal aspectRatio() const { return screenWidth_ / static_cast<qreal>(screenHeight_); }
 	/*! Returns the width (in pixels) of the Camera screen.
 
 	Set using setScreenWidthAndHeight(). This value is automatically fitted to the QGLViewer's
@@ -232,7 +194,7 @@ public:
 	window dimensions when the Camera is attached to a QGLViewer. See also QGLWidget::height() */
 	int screenHeight() const { return screenHeight_; }
 	void getViewport(GLint viewport[4]) const;
-	float pixelGLRatio(const Vec& position) const;
+	qreal pixelGLRatio(const Vec& position) const;
 
 	/*! Returns the coefficient which is used to set zNear() when the Camera is inside the sphere
 	defined by sceneCenter() and zClippingCoefficient() * sceneRadius().
@@ -245,7 +207,7 @@ public:
 	clipping of very close objects at the expense of a worst Z precision.
 
 	Only meaningful when Camera type is Camera::PERSPECTIVE. */
-	float zNearCoefficient() const { return zNearCoef_; }
+	qreal zNearCoefficient() const { return zNearCoef_; }
 	/*! Returns the coefficient used to position the near and far clipping planes.
 
 	The near (resp. far) clipping plane is positioned at a distance equal to zClippingCoefficient() *
@@ -264,24 +226,24 @@ public:
 	\attention When QGLViewer::cameraPathAreEdited(), this value is set to 5.0 so that the Camera
 	paths are not clipped. The previous zClippingCoefficient() value is restored back when you leave
 	this mode. */
-	float zClippingCoefficient() const { return zClippingCoef_; }
+	qreal zClippingCoefficient() const { return zClippingCoef_; }
 
-	virtual float zNear() const;
-	virtual float zFar()  const;
+	virtual qreal zNear() const;
+	virtual qreal zFar()  const;
 	virtual void getOrthoWidthHeight(GLdouble& halfWidth, GLdouble& halfHeight) const;
 	void getFrustumPlanesCoefficients(GLdouble coef[6][4]) const;
 
 public Q_SLOTS:
 	void setType(Type type);
 
-	void setFieldOfView(float fov);
+	void setFieldOfView(qreal fov);
 
 	/*! Sets the horizontalFieldOfView() of the Camera (in radians).
 
 	horizontalFieldOfView() and fieldOfView() are linked by the aspectRatio(). This method actually
 	calls setFieldOfView(( 2.0 * atan (tan(hfov / 2.0) / aspectRatio()) )) so that a call to
 	horizontalFieldOfView() returns the expected value. */
-	void setHorizontalFieldOfView(float hfov) { setFieldOfView( 2.0 * atan (tan(hfov / 2.0) / aspectRatio()) ); }
+	void setHorizontalFieldOfView(qreal hfov) { setFieldOfView( 2.0 * atan (tan(hfov / 2.0) / aspectRatio()) ); }
 
 	void setFOVToFitScene();
 
@@ -297,13 +259,13 @@ public Q_SLOTS:
 	\note If you absolutely need an aspectRatio() that does not correspond to your viewer's window
 	dimensions, overload loadProjectionMatrix() or multiply the created GL_PROJECTION matrix by a
 	scaled diagonal matrix in your QGLViewer::draw() method. */
-	void setAspectRatio(float aspect) { setScreenWidthAndHeight(int(100.0*aspect), 100); }
+	void setAspectRatio(qreal aspect) { setScreenWidthAndHeight(int(100.0*aspect), 100); }
 
 	void setScreenWidthAndHeight(int width, int height);
 	/*! Sets the zNearCoefficient() value. */
-	void setZNearCoefficient(float coef) { zNearCoef_ = coef; projectionMatrixIsUpToDate_ = false; }
+	void setZNearCoefficient(qreal coef) { zNearCoef_ = coef; projectionMatrixIsUpToDate_ = false; }
 	/*! Sets the zClippingCoefficient() value. */
-	void setZClippingCoefficient(float coef) { zClippingCoef_ = coef; projectionMatrixIsUpToDate_ = false; }
+	void setZClippingCoefficient(qreal coef) { zClippingCoef_ = coef; projectionMatrixIsUpToDate_ = false; }
 	//@}
 
 
@@ -319,7 +281,7 @@ public:
 
 	Note that QGLViewer::sceneRadius() (resp. QGLViewer::setSceneRadius()) simply call this method
 	(resp. setSceneRadius()) on its associated QGLViewer::camera(). */
-	float sceneRadius() const { return sceneRadius_; }
+	qreal sceneRadius() const { return sceneRadius_; }
 
 	/*! Returns the position of the scene center, defined in the world coordinate system.
 
@@ -330,32 +292,35 @@ public:
 	Default value is (0,0,0) (world origin). Use setSceneCenter() to change it. See also
 	setSceneBoundingBox().
 
-	Note that QGLViewer::sceneCenter() (resp. QGLViewer::setSceneCenter()) simply call this method
+	Note that QGLViewer::sceneCenter() (resp. QGLViewer::setSceneCenter()) simply calls this method
 	(resp. setSceneCenter()) on its associated QGLViewer::camera(). */
 	Vec sceneCenter() const { return sceneCenter_; }
-	float distanceToSceneCenter() const;
+	qreal distanceToSceneCenter() const;
 
 public Q_SLOTS:
-	void setSceneRadius(float radius);
+	void setSceneRadius(qreal radius);
 	void setSceneCenter(const Vec& center);
 	bool setSceneCenterFromPixel(const QPoint& pixel);
 	void setSceneBoundingBox(const Vec& min, const Vec& max);
 	//@}
 
 
-	/*! @name Revolve Around Point */
+	/*! @name Pivot Point */
 	//@{
 public Q_SLOTS:
-	void setRevolveAroundPoint(const Vec& rap);
-	bool setRevolveAroundPointFromPixel(const QPoint& pixel);
+	void setPivotPoint(const Vec& point);
+	bool setPivotPointFromPixel(const QPoint& pixel);
 
 public:
-	/*! The point the Camera revolves around with the QGLViewer::ROTATE mouse binding. Defined in world coordinate system.
+	Vec pivotPoint() const;
 
-	Default value is the sceneCenter().
-
-	\attention setSceneCenter() changes this value. */
-	Vec revolveAroundPoint() const { return frame()->revolveAroundPoint(); }
+#ifndef DOXYGEN
+public Q_SLOTS:
+	void setRevolveAroundPoint(const Vec& point);
+	bool setRevolveAroundPointFromPixel(const QPoint& pixel);
+public:
+	Vec revolveAroundPoint() const;
+#endif
 	//@}
 
 
@@ -375,15 +340,15 @@ public Q_SLOTS:
 	/*! @name KeyFramed paths */
 	//@{
 public:
-	KeyFrameInterpolator* keyFrameInterpolator(int i) const;
+	KeyFrameInterpolator* keyFrameInterpolator(unsigned int i) const;
 
 public Q_SLOTS:
-	void setKeyFrameInterpolator(int i, KeyFrameInterpolator* const kfi);
+	void setKeyFrameInterpolator(unsigned int i, KeyFrameInterpolator* const kfi);
 
-	virtual void addKeyFrameToPath(int i);
-	virtual void playPath(int i);
-	virtual void deletePath(int i);
-	virtual void resetPath(int i);
+	virtual void addKeyFrameToPath(unsigned int i);
+	virtual void playPath(unsigned int i);
+	virtual void deletePath(unsigned int i);
+	virtual void resetPath(unsigned int i);
 	virtual void drawAllPaths();
 	//@}
 
@@ -399,44 +364,33 @@ public:
 	virtual void loadProjectionMatrixStereo(bool leftBuffer=true) const;
 	virtual void loadModelViewMatrixStereo(bool leftBuffer=true) const;
 
-	void getProjectionMatrix(GLdouble m[16]) const;
-	void getModelViewMatrix(GLdouble m[16]) const;
-	void getModelViewProjectionMatrix(GLdouble m[16]) const;
-
-#ifndef DOXYGEN
-	// Required for windows which otherwise silently fills
 	void getProjectionMatrix(GLfloat m[16]) const;
+	void getProjectionMatrix(GLdouble m[16]) const;
+
 	void getModelViewMatrix(GLfloat m[16]) const;
-#endif
+	void getModelViewMatrix(GLdouble m[16]) const;
+
+	void getModelViewProjectionMatrix(GLfloat m[16]) const;
+	void getModelViewProjectionMatrix(GLdouble m[16]) const;
 	//@}
 
 
 	/*! @name Drawing */
 	//@{
 #ifndef DOXYGEN
-	static void drawCamera(float scale=1.0, float aspectRatio=1.33, float fieldOfView=M_PI/4.0);
+	static void drawCamera(qreal scale=1.0, qreal aspectRatio=1.33, qreal fieldOfView=qreal(M_PI)/4.0);
 #endif
-	virtual void draw(bool drawFarPlane=true, float scale=1.0) const;
+	virtual void draw(bool drawFarPlane=true, qreal scale=1.0) const;
 	//@}
 
 
 	/*! @name World to Camera coordinate systems conversions */
 	//@{
 public:
-	/*! Returns the Camera frame coordinates of a point \p src defined in world coordinates.
-
-	worldCoordinatesOf() performs the inverse transformation.
-
-	Note that the point coordinates are simply converted in a different coordinate system. They are
-	not projected on screen. Use projectedCoordinatesOf() for that. */
-	Vec cameraCoordinatesOf(const Vec& src) const { return frame()->coordinatesOf(src); }
-	/*! Returns the world coordinates of the point whose position \p src is defined in the Camera
-	coordinate system.
-
-	cameraCoordinatesOf() performs the inverse transformation. */
-	Vec worldCoordinatesOf(const Vec& src) const { return frame()->inverseCoordinatesOf(src); }
-	void getCameraCoordinatesOf(const float src[3], float res[3]) const;
-	void getWorldCoordinatesOf(const float src[3], float res[3]) const;
+	Vec cameraCoordinatesOf(const Vec& src) const;
+	Vec worldCoordinatesOf(const Vec& src) const;
+	void getCameraCoordinatesOf(const qreal src[3], qreal res[3]) const;
+	void getWorldCoordinatesOf(const qreal src[3], qreal res[3]) const;
 	//@}
 
 
@@ -445,8 +399,8 @@ public:
 public:
 	Vec projectedCoordinatesOf(const Vec& src, const Frame* frame=NULL) const;
 	Vec unprojectedCoordinatesOf(const Vec& src, const Frame* frame=NULL) const;
-	void getProjectedCoordinatesOf(const float src[3], float res[3], const Frame* frame=NULL) const;
-	void getUnprojectedCoordinatesOf(const float src[3], float res[3], const Frame* frame=NULL) const;
+	void getProjectedCoordinatesOf(const qreal src[3], qreal res[3], const Frame* frame=NULL) const;
+	void getUnprojectedCoordinatesOf(const qreal src[3], qreal res[3], const Frame* frame=NULL) const;
 	void convertClickToLine(const QPoint& pixel, Vec& orig, Vec& dir) const;
 	Vec pointUnderPixel(const QPoint& pixel, bool& found) const;
 	//@}
@@ -455,19 +409,9 @@ public:
 	/*! @name Fly speed */
 	//@{
 public:
-	/*! Returns the fly speed of the Camera.
-
-	Simply returns frame()->flySpeed(). See the ManipulatedCameraFrame::flySpeed() documentation.
-	This value is only meaningful when the MouseAction bindings is QGLViewer::MOVE_FORWARD or
-	QGLViewer::MOVE_BACKWARD.
-
-	Set to 1% of the sceneRadius() by setSceneRadius(). See also setFlySpeed(). */
-	float flySpeed() const { return frame()->flySpeed(); }
+	qreal flySpeed() const;
 public Q_SLOTS:
-	/*! Sets the Camera flySpeed().
-
-	\attention This value is modified by setSceneRadius(). */
-	void setFlySpeed(float speed) { frame()->setFlySpeed(speed); }
+	void setFlySpeed(qreal speed);
 	//@}
 
 
@@ -478,7 +422,7 @@ public:
 
 	loadProjectionMatrixStereo() uses this value to define the Camera offset and frustum. See
 	setIODistance(). */
-	float IODistance() const { return IODistance_; }
+	qreal IODistance() const { return IODistance_; }
 
 	/*! Returns the physical distance between the user's eyes and the screen (in meters).
 
@@ -487,13 +431,13 @@ public:
 	units.
 
 	This is a helper function. It simply returns physicalScreenWidth() / 2.0 / tan(horizontalFieldOfView() / 2.0); */
-	float physicalDistanceToScreen() const { return physicalScreenWidth() / 2.0f / tan(horizontalFieldOfView() / 2.0); }
+	qreal physicalDistanceToScreen() const { return physicalScreenWidth() / 2.0 / tan(horizontalFieldOfView() / 2.0); }
 
 	/*! Returns the physical screen width, in meters. Default value is 0.5m (average monitor width).
 
 	Used for stereo display only (see loadModelViewMatrixStereo() and loadProjectionMatrixStereo()).
 	Set using setPhysicalScreenWidth(). */
-	float physicalScreenWidth() const { return physicalScreenWidth_; }
+	qreal physicalScreenWidth() const { return physicalScreenWidth_; }
 
 	/*! Returns the focus distance used by stereo display, expressed in OpenGL units.
 
@@ -505,21 +449,21 @@ public:
 	\attention This value is modified by QGLViewer::setSceneRadius(), setSceneRadius() and
 	setFieldOfView(). When one of these values is modified, focusDistance() is set to sceneRadius()
 	/ tan(fieldOfView()/2), which provides good results. */
-	float focusDistance() const { return focusDistance_; }
+	qreal focusDistance() const { return focusDistance_; }
 public Q_SLOTS:
 	/*! Sets the IODistance(). */
-	void setIODistance(float distance) { IODistance_ = distance; }
+	void setIODistance(qreal distance) { IODistance_ = distance; }
 
 #ifndef DOXYGEN
 	/*! This method is deprecated. Use setPhysicalScreenWidth() instead. */
-	void setPhysicalDistanceToScreen(float distance) { Q_UNUSED(distance); qWarning("setPhysicalDistanceToScreen is deprecated, use setPhysicalScreenWidth instead"); }
+	void setPhysicalDistanceToScreen(qreal distance) { Q_UNUSED(distance); qWarning("setPhysicalDistanceToScreen is deprecated, use setPhysicalScreenWidth instead"); }
 #endif
 
 	/*! Sets the physical screen (monitor or projected wall) width (in meters). */
-	void setPhysicalScreenWidth(float width) { physicalScreenWidth_ = width; }
+	void setPhysicalScreenWidth(qreal width) { physicalScreenWidth_ = width; }
 
 	/*! Sets the focusDistance(), in OpenGL scene units. */
-	void setFocusDistance(float distance) { focusDistance_ = distance; }
+	void setFocusDistance(qreal distance) { focusDistance_ = distance; }
 	//@}
 
 
@@ -541,12 +485,12 @@ private:
 
 	// C a m e r a   p a r a m e t e r s
 	int screenWidth_, screenHeight_;  // size of the window, in pixels
-	float fieldOfView_; // in radians
+	qreal fieldOfView_; // in radians
 	Vec sceneCenter_;
-	float sceneRadius_; // OpenGL units
-	float zNearCoef_;
-	float zClippingCoef_;
-	float orthoCoef_;
+	qreal sceneRadius_; // OpenGL units
+	qreal zNearCoef_;
+	qreal zClippingCoef_;
+	qreal orthoCoef_;
 	Type type_; // PERSPECTIVE or ORTHOGRAPHIC
 	mutable GLdouble modelViewMatrix_[16]; // Buffered model view matrix.
 	mutable bool modelViewMatrixIsUpToDate_;
@@ -554,12 +498,12 @@ private:
 	mutable bool projectionMatrixIsUpToDate_;
 
 	// S t e r e o   p a r a m e t e r s
-	float IODistance_;		     // inter-ocular distance, in meters
-	float focusDistance_;	     // in scene units
-	float physicalScreenWidth_;	     // in meters
+	qreal IODistance_;		     // inter-ocular distance, in meters
+	qreal focusDistance_;	     // in scene units
+	qreal physicalScreenWidth_;	     // in meters
 
 	// P o i n t s   o f   V i e w s   a n d   K e y F r a m e s
-	QMap<int, KeyFrameInterpolator*> kfi_;
+	QMap<unsigned int, KeyFrameInterpolator*> kfi_;
 	KeyFrameInterpolator* interpolationKfi_;
 };
 
