@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -23,13 +23,8 @@
 #ifndef QGLVIEWER_FRAME_H
 #define QGLVIEWER_FRAME_H
 
-#if QT_VERSION >= 0x040000
-# include <QObject>
-# include <QString>
-#else
-# include <qobject.h>
-# include <qstring.h>
-#endif
+#include <QObject>
+#include <QString>
 
 #include "constraint.h"
 // #include "GL/gl.h" is now included in config.h for ease of configuration
@@ -158,8 +153,8 @@ Q_SIGNALS:
 	will prevent you from modifying the Frame \e inside your QGLViewer::draw() method as it would
 	result in an infinite loop. However, QGLViewer::draw() should not modify the scene.
 
-	\note For efficiency reasons, this signal is emitted even if the Frame is not actually modified, for
-	instance with translate(Vec(0,0,0)) or setPosition(position()). */
+	\note Note that this signal might be emitted even if the Frame is not actually modified, for
+	instance after a translate(Vec(0,0,0)) or a setPosition(position()). */
 	void modified();
 
 	/*! This signal is emitted when the Frame is interpolated by a KeyFrameInterpolator.
@@ -177,23 +172,21 @@ public:
 	Frame(const Vec& position, const Quaternion& orientation);
 
 	void setPosition(const Vec& position);
-	void setPosition(float x, float y, float z);
+	void setPosition(qreal x, qreal y, qreal z);
 	void setPositionWithConstraint(Vec& position);
 
 	void setOrientation(const Quaternion& orientation);
-	void setOrientation(double q0, double q1, double q2, double q3);
+	void setOrientation(qreal q0, qreal q1, qreal q2, qreal q3);
 	void setOrientationWithConstraint(Quaternion& orientation);
 
 	void setPositionAndOrientation(const Vec& position, const Quaternion& orientation);
 	void setPositionAndOrientationWithConstraint(Vec& position, Quaternion& orientation);
 
-	/*! Returns the position of the Frame, defined in the world coordinate system. See also
-	  orientation(), setPosition() and translation(). */
-	Vec position() const { return inverseCoordinatesOf(Vec(0.0,0.0,0.0)); }
+	Vec position() const;
 	Quaternion orientation() const;
 
-	void getPosition(float& x, float& y, float& z) const;
-	void getOrientation(double& q0, double& q1, double& q2, double& q3) const;
+	void getPosition(qreal& x, qreal& y, qreal& z) const;
+	void getOrientation(qreal& q0, qreal& q1, qreal& q2, qreal& q3) const;
 	//@}
 
 
@@ -206,7 +199,7 @@ public:
 	Use setPosition() to define the world coordinates position(). Use
 	setTranslationWithConstraint() to take into account the potential constraint() of the Frame. */
 	void setTranslation(const Vec& translation) { t_ = translation; Q_EMIT modified(); }
-	void setTranslation(float x, float y, float z);
+	void setTranslation(qreal x, qreal y, qreal z);
 	void setTranslationWithConstraint(Vec& translation);
 
 	/*! Set the current rotation Quaternion. See rotation() and the different Quaternion
@@ -220,7 +213,7 @@ public:
 	 constraint() of the Frame is not taken into account, use setRotationWithConstraint()
 	 instead. */
 	void setRotation(const Quaternion& rotation) { q_ = rotation; Q_EMIT modified(); }
-	void setRotation(double q0, double q1, double q2, double q3);
+	void setRotation(qreal q0, qreal q1, qreal q2, qreal q3);
 	void setRotationWithConstraint(Quaternion& rotation);
 
 	void setTranslationAndRotation(const Vec& translation, const Quaternion& rotation);
@@ -243,8 +236,8 @@ public:
 	/*! Returns the current Quaternion orientation. See setRotation(). */
 	Quaternion rotation() const { return q_; }
 
-	void getTranslation(float& x, float& y, float& z) const;
-	void getRotation(double& q0, double& q1, double& q2, double& q3) const;
+	void getTranslation(qreal& x, qreal& y, qreal& z) const;
+	void getRotation(qreal& q0, qreal& q1, qreal& q2, qreal& q3) const;
 	//@}
 
 public:
@@ -278,20 +271,20 @@ public:
 	void translate(const Vec& t);
 	// Some compilers complain about "overloading cannot distinguish from previous declaration"
 	// Simply comment out the following method and its associated implementation
-	void translate(float x, float y, float z);
-	void translate(float& x, float& y, float& z);
+	void translate(qreal x, qreal y, qreal z);
+	void translate(qreal& x, qreal& y, qreal& z);
 
 	void rotate(Quaternion& q);
 	void rotate(const Quaternion& q);
 	// Some compilers complain about "overloading cannot distinguish from previous declaration"
 	// Simply comment out the following method and its associated implementation
-	void rotate(double q0, double q1, double q2, double q3);
-	void rotate(double& q0, double& q1, double& q2, double& q3);
+	void rotate(qreal q0, qreal q1, qreal q2, qreal q3);
+	void rotate(qreal& q0, qreal& q1, qreal& q2, qreal& q3);
 
 	void rotateAroundPoint(Quaternion& rotation, const Vec& point);
 	void rotateAroundPoint(const Quaternion& rotation, const Vec& point);
 
-	void alignWithFrame(const Frame* const frame, bool move=false, float threshold=0.85f);
+	void alignWithFrame(const Frame* const frame, bool move=false, qreal threshold=0.0);
 	void projectOnLine(const Vec& origin, const Vec& direction);
 	//@}
 
@@ -305,12 +298,12 @@ public:
 	Vec coordinatesOfIn(const Vec& src, const Frame* const in) const;
 	Vec coordinatesOfFrom(const Vec& src, const Frame* const from) const;
 
-	void getCoordinatesOf(const float src[3], float res[3]) const;
-	void getInverseCoordinatesOf(const float src[3], float res[3]) const;
-	void getLocalCoordinatesOf(const float src[3], float res[3]) const;
-	void getLocalInverseCoordinatesOf(const float src[3], float res[3]) const;
-	void getCoordinatesOfIn(const float src[3], float res[3], const Frame* const in) const;
-	void getCoordinatesOfFrom(const float src[3], float res[3], const Frame* const from) const;
+	void getCoordinatesOf(const qreal src[3], qreal res[3]) const;
+	void getInverseCoordinatesOf(const qreal src[3], qreal res[3]) const;
+	void getLocalCoordinatesOf(const qreal src[3], qreal res[3]) const;
+	void getLocalInverseCoordinatesOf(const qreal src[3], qreal res[3]) const;
+	void getCoordinatesOfIn(const qreal src[3], qreal res[3], const Frame* const in) const;
+	void getCoordinatesOfFrom(const qreal src[3], qreal res[3], const Frame* const from) const;
 	//@}
 
 	/*! @name Coordinate system transformation of vectors */
@@ -343,12 +336,12 @@ public:
 	Vec transformOfIn(const Vec& src, const Frame* const in) const;
 	Vec transformOfFrom(const Vec& src, const Frame* const from) const;
 
-	void getTransformOf(const float src[3], float res[3]) const;
-	void getInverseTransformOf(const float src[3], float res[3]) const;
-	void getLocalTransformOf(const float src[3], float res[3]) const;
-	void getLocalInverseTransformOf(const float src[3], float res[3]) const;
-	void getTransformOfIn(const float src[3], float res[3], const Frame* const in) const;
-	void getTransformOfFrom(const float src[3], float res[3], const Frame* const from) const;
+	void getTransformOf(const qreal src[3], qreal res[3]) const;
+	void getInverseTransformOf(const qreal src[3], qreal res[3]) const;
+	void getLocalTransformOf(const qreal src[3], qreal res[3]) const;
+	void getLocalInverseTransformOf(const qreal src[3], qreal res[3]) const;
+	void getTransformOfIn(const qreal src[3], qreal res[3], const Frame* const in) const;
+	void getTransformOfFrom(const qreal src[3], qreal res[3], const Frame* const from) const;
 	//@}
 
 
