@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -22,14 +22,13 @@
 
 #include "domUtils.h"
 #include "manipulatedFrame.h"
+#include "manipulatedCameraFrame.h"
 #include "qglviewer.h"
 #include "camera.h"
 
 #include <cstdlib>
 
-#if QT_VERSION >= 0x040000
-# include <QMouseEvent>
-#endif
+#include <QMouseEvent>
 
 using namespace qglviewer;
 using namespace std;
@@ -45,10 +44,11 @@ ManipulatedFrame::ManipulatedFrame()
 	: action_(QGLViewer::NO_MOUSE_ACTION), keepsGrabbingMouse_(false)
 {
 	// #CONNECTION# initFromDOMElement and accessor docs
-	setRotationSensitivity(1.0f);
-	setTranslationSensitivity(1.0f);
-	setSpinningSensitivity(0.3f);
-	setWheelSensitivity(1.0f);
+	setRotationSensitivity(1.0);
+	setTranslationSensitivity(1.0);
+	setSpinningSensitivity(0.3);
+	setWheelSensitivity(1.0);
+	setZoomSensitivity(1.0);
 
 	isSpinning_ = false;
 	previousConstraint_ = NULL;
@@ -65,10 +65,12 @@ ManipulatedFrame& ManipulatedFrame::operator=(const ManipulatedFrame& mf)
 	setTranslationSensitivity(mf.translationSensitivity());
 	setSpinningSensitivity(mf.spinningSensitivity());
 	setWheelSensitivity(mf.wheelSensitivity());
+	setZoomSensitivity(mf.zoomSensitivity());
 
 	mouseSpeed_ = 0.0;
 	dirIsFixed_ = false;
 	keepsGrabbingMouse_ = false;
+	action_ = QGLViewer::NO_MOUSE_ACTION;
 
 	return *this;
 }
@@ -119,6 +121,7 @@ QDomElement ManipulatedFrame::domElement(const QString& name, QDomDocument& docu
 	mp.setAttribute("transSens", QString::number(translationSensitivity()));
 	mp.setAttribute("spinSens", QString::number(spinningSensitivity()));
 	mp.setAttribute("wheelSens", QString::number(wheelSensitivity()));
+	mp.setAttribute("zoomSens", QString::number(zoomSensitivity()));
 	e.appendChild(mp);
 	return e;
 }
@@ -146,10 +149,11 @@ void ManipulatedFrame::initFromDOMElement(const QDomElement& element)
 		if (child.tagName() == "ManipulatedParameters")
 		{
 			// #CONNECTION# constructor default values and accessor docs
-			setRotationSensitivity   (DomUtils::floatFromDom(child, "rotSens",   1.0f));
-			setTranslationSensitivity(DomUtils::floatFromDom(child, "transSens", 1.0f));
-			setSpinningSensitivity   (DomUtils::floatFromDom(child, "spinSens",  0.3f));
-			setWheelSensitivity      (DomUtils::floatFromDom(child, "wheelSens", 1.0f));
+			setRotationSensitivity   (DomUtils::qrealFromDom(child, "rotSens",   1.0));
+			setTranslationSensitivity(DomUtils::qrealFromDom(child, "transSens", 1.0));
+			setSpinningSensitivity   (DomUtils::qrealFromDom(child, "spinSens",  0.3));
+			setWheelSensitivity      (DomUtils::qrealFromDom(child, "wheelSens", 1.0));
+			setZoomSensitivity       (DomUtils::qrealFromDom(child, "zoomSens", 1.0));
 		}
 		child = child.nextSibling().toElement();
 	}
@@ -217,18 +221,18 @@ void ManipulatedFrame::startAction(int ma, bool withConstraint)
 
 	switch (action_)
 	{
-	case QGLViewer::ROTATE:
-	case QGLViewer::SCREEN_ROTATE:
-		mouseSpeed_ = 0.0;
-		stopSpinning();
-		break;
+		case QGLViewer::ROTATE:
+		case QGLViewer::SCREEN_ROTATE:
+			mouseSpeed_ = 0.0;
+			stopSpinning();
+			break;
 
-	case QGLViewer::SCREEN_TRANSLATE:
-		dirIsFixed_ = false;
-		break;
+		case QGLViewer::SCREEN_TRANSLATE:
+			dirIsFixed_ = false;
+			break;
 
-	default:
-		break;
+		default:
+			break;
 	}
 }
 
@@ -237,7 +241,7 @@ use mouse speed. Currently used to trigger spinning in mouseReleaseEvent(). */
 void ManipulatedFrame::computeMouseSpeed(const QMouseEvent* const e)
 {
 	const QPoint delta = (e->pos() - prevPos_);
-	const float dist = sqrt(static_cast<float>(delta.x()*delta.x() + delta.y()*delta.y()));
+	const qreal dist = sqrt(qreal(delta.x()*delta.x() + delta.y()*delta.y()));
 	delay_ = last_move_time.restart();
 	if (delay_ == 0)
 		// Less than a millisecond: assume delay = 1ms
@@ -267,6 +271,29 @@ int ManipulatedFrame::mouseOriginalDirection(const QMouseEvent* const e)
 	else
 		return 0;
 }
+
+qreal ManipulatedFrame::deltaWithPrevPos(QMouseEvent* const event, Camera* const camera) const {
+	qreal dx = qreal(event->x() - prevPos_.x()) / camera->screenWidth();
+	qreal dy = qreal(event->y() - prevPos_.y()) / camera->screenHeight();
+
+	qreal value = fabs(dx) > fabs(dy) ? dx : dy;
+	return value * zoomSensitivity();
+}
+
+qreal ManipulatedFrame::wheelDelta(const QWheelEvent* event) const {
+	static const qreal WHEEL_SENSITIVITY_COEF = 8E-4;
+	return event->delta() * wheelSensitivity() * WHEEL_SENSITIVITY_COEF;
+}
+
+void ManipulatedFrame::zoom(qreal delta, const Camera * const camera) {
+	Vec trans(0.0, 0.0, (camera->position() - position()).norm() * delta);
+
+	trans = camera->frame()->orientation().rotate(trans);
+	if (referenceFrame())
+		trans = referenceFrame()->transformOf(trans);
+	translate(trans);
+}
+
 #endif // DOXYGEN
 
 /*! Initiates the ManipulatedFrame mouse manipulation.
@@ -284,10 +311,8 @@ void ManipulatedFrame::mousePressEvent(QMouseEvent* const event, Camera* const c
 
 	// #CONNECTION setMouseBinding
 	// action_ should no longer possibly be NO_MOUSE_ACTION since this value is not inserted in mouseBinding_
-	//#if QT_VERSION >= 0x030000
 	//if (action_ == QGLViewer::NO_MOUSE_ACTION)
 	//event->ignore();
-	//#endif
 
 	prevPos_ = pressPos_ = event->pos();
 }
@@ -305,115 +330,118 @@ void ManipulatedFrame::mouseMoveEvent(QMouseEvent* const event, Camera* const ca
 {
 	switch (action_)
 	{
-	case QGLViewer::TRANSLATE:
-	{
-		const QPoint delta = event->pos() - prevPos_;
-		Vec trans(static_cast<float>(delta.x()), static_cast<float>(-delta.y()), 0.0);
-		// Scale to fit the screen mouse displacement
-		switch (camera->type())
+		case QGLViewer::TRANSLATE:
 		{
-		case Camera::PERSPECTIVE :
-			trans *= 2.0 * tan(camera->fieldOfView()/2.0) * fabs((camera->frame()->coordinatesOf(position())).z) / camera->screenHeight();
-			break;
-		case Camera::ORTHOGRAPHIC :
-		{
-			GLdouble w,h;
-			camera->getOrthoWidthHeight(w, h);
-			trans[0] *= 2.0 * w / camera->screenWidth();
-			trans[1] *= 2.0 * h / camera->screenHeight();
+			const QPoint delta = event->pos() - prevPos_;
+			Vec trans(delta.x(), -delta.y(), 0.0);
+			// Scale to fit the screen mouse displacement
+			switch (camera->type())
+			{
+				case Camera::PERSPECTIVE :
+					trans *= 2.0 * tan(camera->fieldOfView()/2.0) * fabs((camera->frame()->coordinatesOf(position())).z) / camera->screenHeight();
+					break;
+				case Camera::ORTHOGRAPHIC :
+				{
+					GLdouble w,h;
+					camera->getOrthoWidthHeight(w, h);
+					trans[0] *= 2.0 * w / camera->screenWidth();
+					trans[1] *= 2.0 * h / camera->screenHeight();
+					break;
+				}
+			}
+			// Transform to world coordinate system.
+			trans = camera->frame()->orientation().rotate(translationSensitivity()*trans);
+			// And then down to frame
+			if (referenceFrame()) trans = referenceFrame()->transformOf(trans);
+			translate(trans);
 			break;
 		}
-		}
-		// Transform to world coordinate system.
-		trans = camera->frame()->orientation().rotate(translationSensitivity()*trans);
-		// And then down to frame
-		if (referenceFrame()) trans = referenceFrame()->transformOf(trans);
-		translate(trans);
-		break;
-	}
 
-	case QGLViewer::ZOOM:
-	{
-		//#CONNECTION# wheelEvent ZOOM case
-		Vec trans(0.0, 0.0, (camera->position()-position()).norm() * (event->y() - prevPos_.y()) / camera->screenHeight());
-
-		trans = camera->frame()->orientation().rotate(trans);
-		if (referenceFrame())
-			trans = referenceFrame()->transformOf(trans);
-		translate(trans);
-		break;
-	}
-
-	case QGLViewer::SCREEN_ROTATE:
-	{
-		Vec trans = camera->projectedCoordinatesOf(position());
-
-		const double prev_angle = atan2(prevPos_.y()-trans[1], prevPos_.x()-trans[0]);
-		const double      angle = atan2(event->y()-trans[1], event->x()-trans[0]);
-
-		const Vec axis = transformOf(camera->frame()->inverseTransformOf(Vec(0.0, 0.0, -1.0)));
-		Quaternion rot(axis, angle-prev_angle);
-		//#CONNECTION# These two methods should go together (spinning detection and activation)
-		computeMouseSpeed(event);
-		setSpinningQuaternion(rot);
-		spin();
-		break;
-	}
-
-	case QGLViewer::SCREEN_TRANSLATE:
-	{
-		Vec trans;
-		int dir = mouseOriginalDirection(event);
-		if (dir == 1)
-			trans.setValue(static_cast<float>(event->x() - prevPos_.x()), 0.0, 0.0);
-		else if (dir == -1)
-			trans.setValue(0.0, static_cast<float>(prevPos_.y() - event->y()), 0.0);
-
-		switch (camera->type())
+		case QGLViewer::ZOOM:
 		{
-		case Camera::PERSPECTIVE :
-			trans *= 2.0 * tan(camera->fieldOfView()/2.0) * fabs((camera->frame()->coordinatesOf(position())).z) / camera->screenHeight();
-			break;
-		case Camera::ORTHOGRAPHIC :
-		{
-			GLdouble w,h;
-			camera->getOrthoWidthHeight(w, h);
-			trans[0] *= 2.0 * w / camera->screenWidth();
-			trans[1] *= 2.0 * h / camera->screenHeight();
+			zoom(deltaWithPrevPos(event, camera), camera);
 			break;
 		}
+
+		case QGLViewer::SCREEN_ROTATE:
+		{
+			Vec trans = camera->projectedCoordinatesOf(position());
+
+			const qreal prev_angle = atan2(prevPos_.y()-trans[1], prevPos_.x()-trans[0]);
+			const qreal      angle = atan2(event->y()-trans[1], event->x()-trans[0]);
+
+			const Vec axis = transformOf(camera->frame()->inverseTransformOf(Vec(0.0, 0.0, -1.0)));
+			Quaternion rot(axis, angle-prev_angle);
+			//#CONNECTION# These two methods should go together (spinning detection and activation)
+			computeMouseSpeed(event);
+			setSpinningQuaternion(rot);
+			spin();
+			break;
 		}
-		// Transform to world coordinate system.
-		trans = camera->frame()->orientation().rotate(translationSensitivity()*trans);
-		// And then down to frame
-		if (referenceFrame())
-			trans = referenceFrame()->transformOf(trans);
 
-		translate(trans);
-		break;
-	}
+		case QGLViewer::SCREEN_TRANSLATE:
+		{
+			Vec trans;
+			int dir = mouseOriginalDirection(event);
+			if (dir == 1)
+				trans.setValue(event->x() - prevPos_.x(), 0.0, 0.0);
+			else if (dir == -1)
+				trans.setValue(0.0, prevPos_.y() - event->y(), 0.0);
 
-	case QGLViewer::ROTATE:
-	{
-		Vec trans = camera->projectedCoordinatesOf(position());
-		Quaternion rot = deformedBallQuaternion(event->x(), event->y(), trans[0], trans[1], camera);
-		trans = Vec(-rot[0], -rot[1], -rot[2]);
-		trans = camera->frame()->orientation().rotate(trans);
-		trans = transformOf(trans);
-		rot[0] = trans[0];
-		rot[1] = trans[1];
-		rot[2] = trans[2];
-		//#CONNECTION# These two methods should go together (spinning detection and activation)
-		computeMouseSpeed(event);
-		setSpinningQuaternion(rot);
-		spin();
-		break;
-	}
+			switch (camera->type())
+			{
+				case Camera::PERSPECTIVE :
+					trans *= 2.0 * tan(camera->fieldOfView()/2.0) * fabs((camera->frame()->coordinatesOf(position())).z) / camera->screenHeight();
+					break;
+				case Camera::ORTHOGRAPHIC :
+				{
+					GLdouble w,h;
+					camera->getOrthoWidthHeight(w, h);
+					trans[0] *= 2.0 * w / camera->screenWidth();
+					trans[1] *= 2.0 * h / camera->screenHeight();
+					break;
+				}
+			}
+			// Transform to world coordinate system.
+			trans = camera->frame()->orientation().rotate(translationSensitivity()*trans);
+			// And then down to frame
+			if (referenceFrame())
+				trans = referenceFrame()->transformOf(trans);
 
-	case QGLViewer::NO_MOUSE_ACTION:
-		// Possible when the ManipulatedFrame is a MouseGrabber. This method is then called without startAction
-		// because of mouseTracking.
-		break;
+			translate(trans);
+			break;
+		}
+
+		case QGLViewer::ROTATE:
+		{
+			Vec trans = camera->projectedCoordinatesOf(position());
+			Quaternion rot = deformedBallQuaternion(event->x(), event->y(), trans[0], trans[1], camera);
+			trans = Vec(-rot[0], -rot[1], -rot[2]);
+			trans = camera->frame()->orientation().rotate(trans);
+			trans = transformOf(trans);
+			rot[0] = trans[0];
+			rot[1] = trans[1];
+			rot[2] = trans[2];
+			//#CONNECTION# These two methods should go together (spinning detection and activation)
+			computeMouseSpeed(event);
+			setSpinningQuaternion(rot);
+			spin();
+			break;
+		}
+
+		case QGLViewer::MOVE_FORWARD:
+		case QGLViewer::MOVE_BACKWARD:
+		case QGLViewer::LOOK_AROUND:
+		case QGLViewer::ROLL:
+		case QGLViewer::DRIVE:
+		case QGLViewer::ZOOM_ON_REGION:
+			// These MouseAction values make no sense for a manipulatedFrame
+			break;
+
+		case QGLViewer::NO_MOUSE_ACTION:
+			// Possible when the ManipulatedFrame is a MouseGrabber. This method is then called without startAction
+			// because of mouseTracking.
+			break;
 	}
 
 	if (action_ != QGLViewer::NO_MOUSE_ACTION)
@@ -453,36 +481,25 @@ Left button double click aligns the ManipulatedFrame with the \p camera axis (se
  direction. */
 void ManipulatedFrame::mouseDoubleClickEvent(QMouseEvent* const event, Camera* const camera)
 {
-#if QT_VERSION >= 0x040000
 	if (event->modifiers() == Qt::NoModifier)
-#else
-	if (event->state() == Qt::NoButton)
-#endif
 		switch (event->button())
 		{
-		case Qt::LeftButton:  alignWithFrame(camera->frame()); break;
-		case Qt::RightButton: projectOnLine(camera->position(), camera->viewDirection()); break;
-		default: break;
+			case Qt::LeftButton:  alignWithFrame(camera->frame()); break;
+			case Qt::RightButton: projectOnLine(camera->position(), camera->viewDirection()); break;
+			default: break;
 		}
 }
 
 /*! Overloading of MouseGrabber::wheelEvent().
 
 Using the wheel is equivalent to a QGLViewer::ZOOM QGLViewer::MouseAction. See
- QGLViewer::setWheelBinding() and setWheelSensitivity(). */
+ QGLViewer::setWheelBinding(), setWheelSensitivity(). */
 void ManipulatedFrame::wheelEvent(QWheelEvent* const event, Camera* const camera)
 {
 	//#CONNECTION# QGLViewer::setWheelBinding
 	if (action_ == QGLViewer::ZOOM)
 	{
-		const float wheelSensitivityCoef = 8E-4f;
-		Vec trans(0.0, 0.0, -event->delta()*wheelSensitivity()*wheelSensitivityCoef*(camera->position()-position()).norm());
-
-		//#CONNECTION# Cut-pasted from the mouseMoveEvent ZOOM case
-		trans = camera->frame()->orientation().rotate(trans);
-		if (referenceFrame())
-			trans = referenceFrame()->transformOf(trans);
-		translate(trans);
+		zoom(wheelDelta(event), camera);
 		Q_EMIT manipulated();
 	}
 
@@ -500,34 +517,34 @@ void ManipulatedFrame::wheelEvent(QWheelEvent* const event, Camera* const camera
 \arg for a point inside the ball, it is proportional to the euclidean distance to the ball
 \arg for a point outside the ball, it is proportional to the inverse of this distance (tends to
 zero) on the ball, the function is continuous. */
-static float projectOnBall(float x, float y)
+static qreal projectOnBall(qreal x, qreal y)
 {
 	// If you change the size value, change angle computation in deformedBallQuaternion().
-	const float size       = 1.0f;
-	const float size2      = size*size;
-	const float size_limit = size2*0.5;
+	const qreal size       = 1.0;
+	const qreal size2      = size*size;
+	const qreal size_limit = size2*0.5;
 
-	const float d = x*x + y*y;
+	const qreal d = x*x + y*y;
 	return d < size_limit ? sqrt(size2 - d) : size_limit/sqrt(d);
 }
 
 #ifndef DOXYGEN
 /*! Returns a quaternion computed according to the mouse motion. Mouse positions are projected on a
 deformed ball, centered on (\p cx,\p cy). */
-Quaternion ManipulatedFrame::deformedBallQuaternion(int x, int y, float cx, float cy, const Camera* const camera)
+Quaternion ManipulatedFrame::deformedBallQuaternion(int x, int y, qreal cx, qreal cy, const Camera* const camera)
 {
 	// Points on the deformed ball
-	float px = rotationSensitivity() * (prevPos_.x()  - cx) / camera->screenWidth();
-	float py = rotationSensitivity() * (cy - prevPos_.y())  / camera->screenHeight();
-	float dx = rotationSensitivity() * (x - cx)	    / camera->screenWidth();
-	float dy = rotationSensitivity() * (cy - y)	    / camera->screenHeight();
+	qreal px = rotationSensitivity() * (prevPos_.x()  - cx) / camera->screenWidth();
+	qreal py = rotationSensitivity() * (cy - prevPos_.y())  / camera->screenHeight();
+	qreal dx = rotationSensitivity() * (x - cx)	    / camera->screenWidth();
+	qreal dy = rotationSensitivity() * (cy - y)	    / camera->screenHeight();
 
 	const Vec p1(px, py, projectOnBall(px, py));
 	const Vec p2(dx, dy, projectOnBall(dx, dy));
 	// Approximation of rotation angle
 	// Should be divided by the projectOnBall size, but it is 1.0
 	const Vec axis = cross(p2,p1);
-	const float angle = 2.0 * asin(sqrt(axis.squaredNorm() / p1.squaredNorm() / p2.squaredNorm()));
+	const qreal angle = 5.0 * asin(sqrt(axis.squaredNorm() / p1.squaredNorm() / p2.squaredNorm()));
 	return Quaternion(axis, angle);
 }
 #endif // DOXYGEN
