@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -25,26 +25,18 @@
 
 #include "camera.h"
 
-#if QT_VERSION >= 0x040000
-# include <QMap>
-# include <QClipboard>
-#else
-# include <qmap.h>
-# include <qclipboard.h>
-#endif
+#include <QMap>
+#include <QClipboard>
+#include <QTime>
 
 class QTabWidget;
 
 namespace qglviewer {
 class MouseGrabber;
+class ManipulatedFrame;
+class ManipulatedCameraFrame;
 }
 
-#if QT_VERSION < 0x040000
-# define Qt::KeyboardModifiers Qt::ButtonState
-# define Qt::MouseButtons Qt::ButtonState
-# define Qt::WindowFlags Qt::WFlags
-#endif
-
 /*! \brief A versatile 3D OpenGL viewer based on QGLWidget.
 \class QGLViewer qglviewer.h QGLViewer/qglviewer.h
 
@@ -74,7 +66,7 @@ class QGLVIEWER_EXPORT QGLViewer : public QGLWidget
 public:
 	// Complete implementation is provided so that the constructor is defined with QT3_SUPPORT when .h is included.
 	// (Would not be available otherwise since lib is compiled without QT3_SUPPORT).
-#if QT_VERSION < 0x040000 || defined QT3_SUPPORT
+#ifdef QT3_SUPPORT
 	explicit QGLViewer(QWidget* parent=NULL, const char* name=0, const QGLWidget* shareWidget=0, Qt::WindowFlags flags=0)
 		: QGLWidget(parent, name, shareWidget, flags)
 	{ defaultConstructor(); }
@@ -84,14 +76,7 @@ public:
 	{ defaultConstructor(); }
 
 	QGLViewer(QGLContext* context, QWidget* parent, const char* name=0, const QGLWidget* shareWidget=0, Qt::WindowFlags flags=0)
-# if QT_VERSION >= 0x030200
 		: QGLWidget(context, parent, name, shareWidget, flags) {
-# else
-	// MOC_SKIP_BEGIN
-		: QGLWidget(parent, name, shareWidget, flags) {
-		Q_UNUSED(context);
-		// MOC_SKIP_END
-# endif
 		defaultConstructor(); }
 
 #else
@@ -173,7 +158,7 @@ public:
 
 	Use setBackgroundColor() to define and activate a background color.
 
-	\attention Each QColor component is an integer ranging from 0 to 255. This differs from the float
+	\attention Each QColor component is an integer ranging from 0 to 255. This differs from the qreal
 	values used by \c glClearColor() which are in the 0.0-1.0 range. Default value is (51, 51, 51)
 	(dark gray). You may have to change foregroundColor() accordingly.
 
@@ -187,7 +172,7 @@ public:
 	This color is used when FPSIsDisplayed(), gridIsDrawn(), to display the camera paths when the
 	cameraIsEdited().
 
-	\attention Each QColor component is an integer in the range 0-255. This differs from the float
+	\attention Each QColor component is an integer in the range 0-255. This differs from the qreal
 	values used by \c glColor3f() which are in the range 0-1. Default value is (180, 180, 180) (light
 	gray).
 
@@ -218,7 +203,7 @@ public:
 
 	Default value is 1.0. This method is equivalent to camera()->sceneRadius(). See
 	setSceneRadius(). */
-	float sceneRadius() const { return camera()->sceneRadius(); }
+	qreal sceneRadius() const { return camera()->sceneRadius(); }
 	/*! Returns the scene center, defined in world coordinates.
 
 	See sceneRadius() for details.
@@ -226,7 +211,7 @@ public:
 	Default value is (0,0,0). Simply a wrapper for camera()->sceneCenter(). Set using
 	setSceneCenter().
 
-	Do not mismatch this value (that only depends on the scene) with the qglviewer::Camera::revolveAroundPoint(). */
+	Do not mismatch this value (that only depends on the scene) with the qglviewer::Camera::pivotPoint(). */
 	qglviewer::Vec sceneCenter() const { return camera()->sceneCenter(); }
 
 public Q_SLOTS:
@@ -234,11 +219,11 @@ public Q_SLOTS:
 
 		The camera() qglviewer::Camera::flySpeed() is set to 1% of this value by this method. Simple
 		wrapper around camera()->setSceneRadius(). */
-	virtual void setSceneRadius(float radius) { camera()->setSceneRadius(radius); }
+	virtual void setSceneRadius(qreal radius) { camera()->setSceneRadius(radius); }
 
 	/*! Sets the sceneCenter(), defined in world coordinates.
 
-		\attention The qglviewer::Camera::revolveAroundPoint() is set to the sceneCenter() value by this
+		\attention The qglviewer::Camera::pivotPoint() is set to the sceneCenter() value by this
 		method. */
 	virtual void setSceneCenter(const qglviewer::Vec& center) { camera()->setSceneCenter(center); }
 
@@ -323,7 +308,7 @@ public Q_SLOTS:
 	//@{
 public:
 	/*! Returns the aspect ratio of the viewer's widget (width() / height()). */
-	float aspectRatio() const { return static_cast<float>(width())/static_cast<float>(height()); }
+	qreal aspectRatio() const { return width() / static_cast<qreal>(height()); }
 	/*! Returns the current averaged viewer frame rate.
 
 	This value is computed and averaged over 20 successive frames. It only changes every 20 draw()
@@ -334,7 +319,7 @@ public:
 
 	This value is meaningful only when draw() is regularly called, either using a \c QTimer, when
 	animationIsStarted() or when the camera is manipulated with the mouse.  */
-	float currentFPS() { return f_p_s_; }
+	qreal currentFPS() { return f_p_s_; }
 	/*! Returns \c true if the viewer is in fullScreen mode.
 
 	Default value is \c false. Set by setFullScreen() or toggleFullScreen().
@@ -375,27 +360,27 @@ public Q_SLOTS:
 	void toggleCameraMode();
 
 private:
-	bool cameraIsInRevolveMode() const;
+	bool cameraIsInRotateMode() const;
 	//@}
 
 
 	/*! @name Display methods */
 	//@{
 public:
-	static void drawArrow(float length=1.0f, float radius=-1.0f, int nbSubdivisions=12);
-	static void drawArrow(const qglviewer::Vec& from, const qglviewer::Vec& to, float radius=-1.0f, int nbSubdivisions=12);
-	static void drawAxis(float length=1.0f);
-	static void drawGrid(float size=1.0f, int nbSubdivisions=10);
+	static void drawArrow(qreal length=1.0, qreal radius=-1.0, int nbSubdivisions=12);
+	static void drawArrow(const qglviewer::Vec& from, const qglviewer::Vec& to, qreal radius=-1.0, int nbSubdivisions=12);
+	static void drawAxis(qreal length=1.0);
+	static void drawGrid(qreal size=1.0, int nbSubdivisions=10);
 
 	virtual void startScreenCoordinatesSystem(bool upward=false) const;
 	virtual void stopScreenCoordinatesSystem() const;
 
 	void drawText(int x, int y, const QString& text, const QFont& fnt=QFont());
 	void displayMessage(const QString& message, int delay=2000);
-	// void draw3DText(const qglviewer::Vec& pos, const qglviewer::Vec& normal, const QString& string, GLfloat height=0.1f);
+	// void draw3DText(const qglviewer::Vec& pos, const qglviewer::Vec& normal, const QString& string, GLfloat height=0.1);
 
 protected:
-	virtual void drawLight(GLenum light, float scale = 1.0f) const;
+	virtual void drawLight(GLenum light, qreal scale = 1.0) const;
 
 private:
 	void displayFPS();
@@ -458,9 +443,6 @@ protected Q_SLOTS:
 	/*! @name Snapshots */
 	//@{
 public:
-#if QT_VERSION < 0x030000
-	virtual QImage grabFrameBuffer(bool withAlpha=false);
-#endif
 	/*! Returns the snapshot file name used by saveSnapshot().
 
 	This value is used in \p automatic mode (see saveSnapshot()). A dialog is otherwise popped-up to
@@ -519,7 +501,7 @@ public:
 	\note This value has no impact on the images produced in vectorial format. */
 	int snapshotQuality() { return snapshotQuality_; }
 
-	// Qt 2.3 does not support double default value parameters in slots.
+	// Qt 2.3 does not support qreal default value parameters in slots.
 	// Remove "Q_SLOTS" from the following line to compile with Qt 2.3
 public Q_SLOTS:
 	void saveSnapshot(bool automatic=true, bool overwrite=false);
@@ -539,14 +521,14 @@ public Q_SLOTS:
 
 private:
 	bool saveImageSnapshot(const QString& fileName);
-	
+
 #ifndef DOXYGEN
 	/* This class is used internally for screenshot that require tiling (image size size different
 	from window size). Only in that case, is the private tileRegion_ pointer non null.
 	It then contains the current tiled region, which is used by startScreenCoordinatesSystem
 	to adapt the coordinate system. Not using it would result in a tiled drawing of the parts
 	that use startScreenCoordinatesSystem. Also used by scaledFont for same purposes. */
-	class TileRegion { public : double xMin, yMin, xMax, yMax, textScale; };
+	class TileRegion { public : qreal xMin, yMin, xMax, yMax, textScale; };
 #endif
 
 public:
@@ -573,13 +555,9 @@ public:
 		else {
 			QFont f(font);
 			if (f.pixelSize() == -1)
-#if QT_VERSION >= 0x040000
 				f.setPointSizeF(f.pointSizeF() * tileRegion_->textScale);
-#else
-				f.setPointSizeFloat(f.pointSizeFloat() * tileRegion_->textScale);
-#endif
 			else
-				f.setPixelSize(f.pixelSize() * tileRegion_->textScale);
+				f.setPixelSize(int(f.pixelSize() * tileRegion_->textScale));
 			return f;
 		}
 	}
@@ -598,9 +576,9 @@ public:
 
 	Use (0,0) to (bufferTextureMaxU(), bufferTextureMaxV()) texture coordinates to map the entire
 	texture on a quad. */
-	float bufferTextureMaxU() const { return bufferTextureMaxU_; }
+	qreal bufferTextureMaxU() const { return bufferTextureMaxU_; }
 	/*! Same as bufferTextureMaxU(), but for the v texture coordinate. */
-	float bufferTextureMaxV() const { return bufferTextureMaxV_; }
+	qreal bufferTextureMaxV() const { return bufferTextureMaxV_; }
 public Q_SLOTS:
 	void copyBufferToTexture(GLint internalFormat, GLenum format=GL_NONE);
 	//@}
@@ -816,6 +794,7 @@ protected:
 	virtual void mouseDoubleClickEvent(QMouseEvent *);
 	virtual void wheelEvent(QWheelEvent *);
 	virtual void keyPressEvent(QKeyEvent *);
+	virtual void keyReleaseEvent(QKeyEvent *);
 	virtual void timerEvent(QTimerEvent *);
 	virtual void closeEvent(QCloseEvent *);
 	//@}
@@ -918,7 +897,7 @@ void Viewer::drawWithNames() {
 
 	/*! @name Keyboard customization */
 	//@{
-protected:
+public:
 	/*! Defines the different actions that can be associated with a keyboard shortcut using
 	setShortcut().
 
@@ -927,66 +906,62 @@ protected:
 						  SAVE_SCREENSHOT, CAMERA_MODE, FULL_SCREEN, STEREO, ANIMATION, HELP, EDIT_CAMERA,
 						  MOVE_CAMERA_LEFT, MOVE_CAMERA_RIGHT, MOVE_CAMERA_UP, MOVE_CAMERA_DOWN,
 						  INCREASE_FLYSPEED, DECREASE_FLYSPEED, SNAPSHOT_TO_CLIPBOARD };
-public:
+
 	unsigned int shortcut(KeyboardAction action) const;
 #ifndef DOXYGEN
 	// QGLViewer 1.x
-	int keyboardAccelerator(KeyboardAction action) const;
-	Qt::Key keyFrameKey(int index) const;
+	unsigned int keyboardAccelerator(KeyboardAction action) const;
+	Qt::Key keyFrameKey(unsigned int index) const;
 	Qt::KeyboardModifiers playKeyFramePathStateKey() const;
 	// QGLViewer 2.0 without Qt4 support
 	Qt::KeyboardModifiers addKeyFrameStateKey() const;
 	Qt::KeyboardModifiers playPathStateKey() const;
 #endif
-	Qt::Key pathKey(int index) const;
+	Qt::Key pathKey(unsigned int index) const;
 	Qt::KeyboardModifiers addKeyFrameKeyboardModifiers() const;
 	Qt::KeyboardModifiers playPathKeyboardModifiers() const;
 
 public Q_SLOTS:
 	void setShortcut(KeyboardAction action, unsigned int key);
 #ifndef DOXYGEN
-	void setKeyboardAccelerator(KeyboardAction action, int key);
+	void setKeyboardAccelerator(KeyboardAction action, unsigned int key);
 #endif
-	void setKeyDescription(int key, QString description);
+	void setKeyDescription(unsigned int key, QString description);
 	void clearShortcuts();
 
 	// Key Frames shortcut keys
 #ifndef DOXYGEN
 	// QGLViewer 1.x compatibility methods
-	virtual void setKeyFrameKey(int index, int key);
-	virtual void setPlayKeyFramePathStateKey(int buttonState);
+	virtual void setKeyFrameKey(unsigned int index, int key);
+	virtual void setPlayKeyFramePathStateKey(unsigned int buttonState);
 	// QGLViewer 2.0 without Qt4 support
-	virtual void setPlayPathStateKey(int buttonState);
-	virtual void setAddKeyFrameStateKey(int buttonState);
+	virtual void setPlayPathStateKey(unsigned int buttonState);
+	virtual void setAddKeyFrameStateKey(unsigned int buttonState);
 #endif
-	virtual void setPathKey(int key, int index = 0);
+	virtual void setPathKey(int key, unsigned int index = 0);
 	virtual void setPlayPathKeyboardModifiers(Qt::KeyboardModifiers modifiers);
 	virtual void setAddKeyFrameKeyboardModifiers(Qt::KeyboardModifiers modifiers);
 	//@}
 
 
+public:
 	/*! @name Mouse customization */
 	//@{
-protected:
 	/*! Defines the different mouse handlers: camera() or manipulatedFrame().
 
-	Used by setMouseBinding(), setMouseBinding(int, ClickAction, bool, int) and setWheelBinding() to
-	define which handler receives the mouse events. */
+	Used by setMouseBinding(), setMouseBinding(Qt::KeyboardModifiers modifiers, Qt::MouseButtons, ClickAction, bool, int)
+	and setWheelBinding() to define which handler receives the mouse events. */
 	enum MouseHandler { CAMERA, FRAME };
 
 	/*! Defines the possible actions that can be binded to a mouse click using
-	setMouseBinding(int,ClickAction,bool,int).
+	setMouseBinding(Qt::KeyboardModifiers, Qt::MouseButtons, ClickAction, bool, int).
 
 	See the <a href="../mouse.html">mouse page</a> for details. */
 	enum ClickAction { NO_CLICK_ACTION, ZOOM_ON_PIXEL, ZOOM_TO_FIT, SELECT, RAP_FROM_PIXEL, RAP_IS_CENTER,
 					   CENTER_FRAME, CENTER_SCENE, SHOW_ENTIRE_SCENE, ALIGN_FRAME, ALIGN_CAMERA };
 
-#ifndef DOXYGEN
-	// So that it can be used in ManipulatedFrame and ManipulatedCameraFrame.
-public:
-#endif
 
-	/*! Defines the possible actions that can be binded to a mouse motion (a click, followed by a
+	/*! Defines the possible actions that can be binded to a mouse action (a click, followed by a
 	mouse displacement).
 
 	These actions may be binded to the camera() or to the manipulatedFrame() (see QGLViewer::MouseHandler) using
@@ -997,45 +972,59 @@ public:
 					   SCREEN_ROTATE, ROLL, DRIVE,
 					   SCREEN_TRANSLATE, ZOOM_ON_REGION };
 
-#ifdef DOXYGEN
-public:
-#endif
-
 #ifndef DOXYGEN
-	MouseAction mouseAction(int state) const;
-	int mouseHandler(int state) const;
+	MouseAction mouseAction(unsigned int state) const;
+	int mouseHandler(unsigned int state) const;
 	int mouseButtonState(MouseHandler handler, MouseAction action, bool withConstraint=true) const;
-	ClickAction clickAction(int state, bool doubleClick, Qt::MouseButtons buttonsBefore) const;
-	void getClickButtonState(ClickAction action, int& state, bool& doubleClick, Qt::MouseButtons& buttonsBefore) const;
+	ClickAction clickAction(unsigned int state, bool doubleClick, Qt::MouseButtons buttonsBefore) const;
+	void getClickButtonState(ClickAction action, unsigned int & state, bool& doubleClick, Qt::MouseButtons& buttonsBefore) const;
+	unsigned int wheelButtonState(MouseHandler handler, MouseAction action, bool withConstraint=true) const;
 #endif
-	MouseAction mouseAction(Qt::KeyboardModifiers modifiers, Qt::MouseButtons buttons) const;
-	int mouseHandler(Qt::KeyboardModifiers modifiers, Qt::MouseButtons buttons) const;
-	ClickAction clickAction(Qt::KeyboardModifiers modifiers, Qt::MouseButtons buttons, bool doubleClick, Qt::MouseButtons buttonsBefore) const;
 
-	Qt::MouseButtons mouseButtons(MouseHandler handler, MouseAction action, bool withConstraint=true) const;
-	Qt::KeyboardModifiers keyboardModifiers(MouseHandler handler, MouseAction action, bool withConstraint=true) const;
-	void getClickActionState(ClickAction action, Qt::KeyboardModifiers& modifiers, Qt::MouseButtons& buttons, bool& doubleClick, Qt::MouseButtons& buttonsBefore) const;
+	MouseAction mouseAction(Qt::Key key, Qt::KeyboardModifiers modifiers, Qt::MouseButton button) const;
+	int mouseHandler(Qt::Key key, Qt::KeyboardModifiers modifiers, Qt::MouseButton button) const;
 
-	MouseAction wheelAction(Qt::KeyboardModifiers modifiers) const;
-	int wheelHandler(Qt::KeyboardModifiers modifiers) const;
-	int wheelButtonState(MouseHandler handler, MouseAction action, bool withConstraint=true) const;
+	void getMouseActionBinding(MouseHandler handler, MouseAction action, bool withConstraint,
+							   Qt::Key& key, Qt::KeyboardModifiers& modifiers, Qt::MouseButton& button) const;
+
+	ClickAction clickAction(Qt::Key key, Qt::KeyboardModifiers modifiers, Qt::MouseButton button,
+							bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton) const;
+
+	void getClickActionBinding(ClickAction action, Qt::Key& key, Qt::KeyboardModifiers& modifiers,
+							   Qt::MouseButton& button, bool& doubleClick, Qt::MouseButtons& buttonsBefore) const;
+
+	MouseAction wheelAction(Qt::Key key, Qt::KeyboardModifiers modifiers) const;
+	int wheelHandler(Qt::Key key, Qt::KeyboardModifiers modifiers) const;
+
+	void getWheelActionBinding(MouseHandler handler, MouseAction action, bool withConstraint,
+							   Qt::Key& key, Qt::KeyboardModifiers& modifiers) const;
 
 public Q_SLOTS:
 #ifndef DOXYGEN
-	void setMouseBinding(int state, MouseHandler handler, MouseAction action, bool withConstraint=true);
-	void setMouseBinding(int state, ClickAction action, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
-	void setMouseBindingDescription(int state, QString description, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
+	void setMouseBinding(unsigned int state, MouseHandler handler, MouseAction action, bool withConstraint=true);
+	void setMouseBinding(unsigned int state, ClickAction action, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
+	void setMouseBindingDescription(unsigned int state, QString description, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
 #endif
-	void setMouseBinding(Qt::KeyboardModifiers modifiers, Qt::MouseButtons buttons, MouseHandler handler, MouseAction action, bool withConstraint=true);
-	void setMouseBinding(Qt::KeyboardModifiers modifiers, Qt::MouseButtons buttons, ClickAction action, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
-	void setMouseBindingDescription(Qt::KeyboardModifiers modifiers, Qt::MouseButtons buttons, QString description, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
 
+	void setMouseBinding(Qt::KeyboardModifiers modifiers, Qt::MouseButton buttons, MouseHandler handler, MouseAction action, bool withConstraint=true);
+	void setMouseBinding(Qt::KeyboardModifiers modifiers, Qt::MouseButton button, ClickAction action, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
 	void setWheelBinding(Qt::KeyboardModifiers modifiers, MouseHandler handler, MouseAction action, bool withConstraint=true);
-	void setHandlerKeyboardModifiers(MouseHandler handler, Qt::KeyboardModifiers modifiers);
+	void setMouseBindingDescription(Qt::KeyboardModifiers modifiers, Qt::MouseButton button, QString description, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
+
+	void setMouseBinding(Qt::Key key, Qt::KeyboardModifiers modifiers, Qt::MouseButton buttons, MouseHandler handler, MouseAction action, bool withConstraint=true);
+	void setMouseBinding(Qt::Key key, Qt::KeyboardModifiers modifiers, Qt::MouseButton button, ClickAction action, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
+	void setWheelBinding(Qt::Key key, Qt::KeyboardModifiers modifiers, MouseHandler handler, MouseAction action, bool withConstraint=true);
+	void setMouseBindingDescription(Qt::Key key, Qt::KeyboardModifiers modifiers, Qt::MouseButton button, QString description, bool doubleClick=false, Qt::MouseButtons buttonsBefore=Qt::NoButton);
+
 	void clearMouseBindings();
+
 #ifndef DOXYGEN
-	void setHandlerStateKey(MouseHandler handler, int buttonState);
-	void setMouseStateKey(MouseHandler handler, int buttonState);
+	MouseAction wheelAction(Qt::KeyboardModifiers modifiers) const;
+	int wheelHandler(Qt::KeyboardModifiers modifiers) const;
+
+	void setHandlerKeyboardModifiers(MouseHandler handler, Qt::KeyboardModifiers modifiers);
+	void setHandlerStateKey(MouseHandler handler, unsigned int buttonState);
+	void setMouseStateKey(MouseHandler handler, unsigned int buttonState);
 #endif
 
 private:
@@ -1093,11 +1082,7 @@ public:
 
 	\attention With Qt version 3, this method returns a \c QPtrList instead. Use a \c QPtrListIterator
 	to iterate on the list instead.*/
-#if QT_VERSION >= 0x040000
 	static const QList<QGLViewer*>& QGLViewerPool() { return QGLViewer::QGLViewerPool_; }
-#else
-	static const QPtrList<QGLViewer>& QGLViewerPool() { return QGLViewer::QGLViewerPool_; }
-#endif
 
 
 	/*! Returns the index of the QGLViewer \p viewer in the QGLViewerPool(). This index in unique and
@@ -1107,11 +1092,7 @@ public:
 	When a QGLViewer is deleted, the QGLViewers' indexes are preserved and NULL is set for that index.
 		When a QGLViewer is created, it is placed in the first available position in that list.
 		Returns -1 if the QGLViewer could not be found (which should not be possible). */
-#if QT_VERSION >= 0x040000
 	static int QGLViewerIndex(const QGLViewer* const viewer) { return QGLViewer::QGLViewerPool_.indexOf(const_cast<QGLViewer*>(viewer)); }
-#else
-	static int QGLViewerIndex(const QGLViewer* const viewer) { return QGLViewer::QGLViewerPool_.findRef(viewer); }
-#endif
 	//@}
 
 #ifndef DOXYGEN
@@ -1145,8 +1126,8 @@ private:
 	// C a m e r a
 	qglviewer::Camera* camera_;
 	bool cameraIsEdited_;
-	float previousCameraZClippingCoefficient_;
-	int previousPathId_; // Double key press recognition
+	qreal previousCameraZClippingCoefficient_;
+	unsigned int previousPathId_; // double key press recognition
 	void connectAllCameraKFIInterpolatedSignals(bool connection=true);
 
 	// C o l o r s
@@ -1170,7 +1151,7 @@ private:
 	QTime fpsTime_;
 	unsigned int fpsCounter_;
 	QString fpsString_;
-	float f_p_s_;
+	qreal f_p_s_;
 
 	// M e s s a g e s
 	QString message_;
@@ -1201,15 +1182,15 @@ private:
 	QString cameraPathKeysString() const;
 	QMap<KeyboardAction, QString> keyboardActionDescription_;
 	QMap<KeyboardAction, unsigned int> keyboardBinding_;
-	QMap<int, QString> keyDescription_;
+	QMap<unsigned int, QString> keyDescription_;
 
 	// K e y   F r a m e s   s h o r t c u t s
-	QMap<Qt::Key, int> pathIndex_;
+	QMap<Qt::Key, unsigned int> pathIndex_;
 	Qt::KeyboardModifiers addKeyFrameKeyboardModifiers_, playPathKeyboardModifiers_;
 
 	// B u f f e r   T e x t u r e
 	GLuint bufferTextureId_;
-	float bufferTextureMaxU_, bufferTextureMaxV_;
+	qreal bufferTextureMaxU_, bufferTextureMaxV_;
 	int bufferTextureWidth_, bufferTextureHeight_;
 	unsigned int previousBufferTextureFormat_;
 	int previousBufferTextureInternalFormat_;
@@ -1225,56 +1206,77 @@ private:
 	// M o u s e   b i n d i n g s
 	struct MouseBindingPrivate {
 		const Qt::KeyboardModifiers modifiers;
-		const Qt::MouseButtons buttons;
+		const Qt::MouseButton button;
+		const Qt::Key key;
 
-		MouseBindingPrivate(Qt::KeyboardModifiers m, Qt::MouseButtons b)
-			: modifiers(m), buttons(b) {}
+		MouseBindingPrivate(Qt::KeyboardModifiers m, Qt::MouseButton b, Qt::Key k)
+			: modifiers(m), button(b), key(k) {}
 
 		// This sort order is used in mouseString() to display sorted mouse bindings
 		bool operator<(const MouseBindingPrivate& mbp) const
 		{
+			if (key != mbp.key)
+				return key < mbp.key;
 			if (modifiers != mbp.modifiers)
 				return modifiers < mbp.modifiers;
-			else
-				return buttons < mbp.buttons;
+			return button < mbp.button;
+		}
+	};
+
+	// W h e e l   b i n d i n g s
+	struct WheelBindingPrivate {
+		const Qt::KeyboardModifiers modifiers;
+		const Qt::Key key;
+
+		WheelBindingPrivate(Qt::KeyboardModifiers m, Qt::Key k)
+			: modifiers(m), key(k) {}
+
+		// This sort order is used in mouseString() to display sorted wheel bindings
+		bool operator<(const WheelBindingPrivate& wbp) const
+		{
+			if (key != wbp.key)
+				return key < wbp.key;
+			return modifiers < wbp.modifiers;
 		}
 	};
 
 	// C l i c k   b i n d i n g s
 	struct ClickBindingPrivate {
 		const Qt::KeyboardModifiers modifiers;
-		const Qt::MouseButtons buttons;
+		const Qt::MouseButton button;
 		const bool doubleClick;
 		const Qt::MouseButtons buttonsBefore; // only defined when doubleClick is true
+		const Qt::Key key;
 
-		ClickBindingPrivate(Qt::KeyboardModifiers m, Qt::MouseButtons b, bool dc, Qt::MouseButtons bb)
-			: modifiers(m), buttons(b), doubleClick(dc), buttonsBefore(bb) {}
+		ClickBindingPrivate(Qt::KeyboardModifiers m, Qt::MouseButton b, bool dc, Qt::MouseButtons bb, Qt::Key k)
+			: modifiers(m), button(b), doubleClick(dc), buttonsBefore(bb), key(k) {}
 
 		// This sort order is used in mouseString() to display sorted mouse bindings
 		bool operator<(const ClickBindingPrivate& cbp) const
 		{
+			if (key != cbp.key)
+				return key < cbp.key;
 			if (buttonsBefore != cbp.buttonsBefore)
 				return buttonsBefore < cbp.buttonsBefore;
-			else
-				if (modifiers != cbp.modifiers)
-					return modifiers < cbp.modifiers;
-				else
-					if (buttons != cbp.buttons)
-						return buttons < cbp.buttons;
-					else
-						return !doubleClick && cbp.doubleClick;
+			if (modifiers != cbp.modifiers)
+				return modifiers < cbp.modifiers;
+			if (button != cbp.button)
+				return button < cbp.button;
+			return doubleClick != cbp.doubleClick;
 		}
 	};
 #endif
 	static QString formatClickActionPrivate(ClickBindingPrivate cbp);
+	static bool isValidShortcutKey(int key);
 
 	QMap<ClickBindingPrivate, QString> mouseDescription_;
 
 	void setDefaultMouseBindings();
 	void performClickAction(ClickAction ca, const QMouseEvent* const e);
 	QMap<MouseBindingPrivate, MouseActionPrivate> mouseBinding_;
-	QMap<Qt::KeyboardModifiers, MouseActionPrivate> wheelBinding_;
+	QMap<WheelBindingPrivate, MouseActionPrivate> wheelBinding_;
 	QMap<ClickBindingPrivate, ClickAction> clickBinding_;
+	Qt::Key currentlyPressedKey_;
 
 	// S n a p s h o t s
 	void initializeSnapshotFormats();
@@ -1284,11 +1286,7 @@ private:
 	TileRegion* tileRegion_;
 
 	// Q G L V i e w e r   p o o l
-#if QT_VERSION >= 0x040000
 	static QList<QGLViewer*> QGLViewerPool_;
-#else
-	static QPtrList<QGLViewer> QGLViewerPool_;
-#endif
 
 	// S t a t e   F i l e
 	QString stateFileName_;
