@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -34,10 +34,10 @@ Note that this rotation is not uniquely defined. The selected axis is usually or
 and \p to, minimizing the rotation angle. This method is robust and can handle small or almost identical vectors. */
 Quaternion::Quaternion(const Vec& from, const Vec& to)
 {
-	const double epsilon = 1E-10f;
+	const qreal epsilon = 1E-10;
 
-	const double fromSqNorm = from.squaredNorm();
-	const double toSqNorm   = to.squaredNorm();
+	const qreal fromSqNorm = from.squaredNorm();
+	const qreal toSqNorm   = to.squaredNorm();
 	// Identity Quaternion when one vector is null
 	if ((fromSqNorm < epsilon) || (toSqNorm < epsilon))
 	{
@@ -47,13 +47,13 @@ Quaternion::Quaternion(const Vec& from, const Vec& to)
 	else
 	{
 		Vec axis = cross(from, to);
-		const double axisSqNorm = axis.squaredNorm();
+		const qreal axisSqNorm = axis.squaredNorm();
 
 		// Aligned vectors, pick any axis, not aligned with from or to
 		if (axisSqNorm < epsilon)
 			axis = from.orthogonalVec();
 
-		double angle = asin(sqrt(axisSqNorm / (fromSqNorm * toSqNorm)));
+		qreal angle = asin(sqrt(axisSqNorm / (fromSqNorm * toSqNorm)));
 
 		if (from*to < 0.0)
 			angle = M_PI-angle;
@@ -75,18 +75,18 @@ Vec Quaternion::inverseRotate(const Vec& v) const
 See also inverseRotate() and operator*(const Quaternion&, const Vec&). */
 Vec Quaternion::rotate(const Vec& v) const
 {
-	const double q00 = 2.0l * q[0] * q[0];
-	const double q11 = 2.0l * q[1] * q[1];
-	const double q22 = 2.0l * q[2] * q[2];
+	const qreal q00 = 2.0 * q[0] * q[0];
+	const qreal q11 = 2.0 * q[1] * q[1];
+	const qreal q22 = 2.0 * q[2] * q[2];
 
-	const double q01 = 2.0l * q[0] * q[1];
-	const double q02 = 2.0l * q[0] * q[2];
-	const double q03 = 2.0l * q[0] * q[3];
+	const qreal q01 = 2.0 * q[0] * q[1];
+	const qreal q02 = 2.0 * q[0] * q[2];
+	const qreal q03 = 2.0 * q[0] * q[3];
 
-	const double q12 = 2.0l * q[1] * q[2];
-	const double q13 = 2.0l * q[1] * q[3];
+	const qreal q12 = 2.0 * q[1] * q[2];
+	const qreal q13 = 2.0 * q[1] * q[3];
 
-	const double q23 = 2.0l * q[2] * q[3];
+	const qreal q23 = 2.0 * q[2] * q[3];
 
 	return Vec((1.0 - q11 - q22)*v[0] + (      q01 - q23)*v[1] + (      q02 + q13)*v[2],
 			(      q01 + q23)*v[0] + (1.0 - q22 - q00)*v[1] + (      q12 - q03)*v[2],
@@ -101,15 +101,15 @@ Vec Quaternion::rotate(const Vec& v) const
 
   setFromRotatedBasis() sets a Quaternion from the three axis of a rotated frame. It actually fills
   the three columns of a matrix with these rotated basis vectors and calls this method. */
-void Quaternion::setFromRotationMatrix(const double m[3][3])
+void Quaternion::setFromRotationMatrix(const qreal m[3][3])
 {
 	// Compute one plus the trace of the matrix
-	const double onePlusTrace = 1.0 + m[0][0] + m[1][1] + m[2][2];
+	const qreal onePlusTrace = 1.0 + m[0][0] + m[1][1] + m[2][2];
 
 	if (onePlusTrace > 1E-5)
 	{
 		// Direct computation
-		const double s = sqrt(onePlusTrace) * 2.0;
+		const qreal s = sqrt(onePlusTrace) * 2.0;
 		q[0] = (m[2][1] - m[1][2]) / s;
 		q[1] = (m[0][2] - m[2][0]) / s;
 		q[2] = (m[1][0] - m[0][1]) / s;
@@ -120,7 +120,7 @@ void Quaternion::setFromRotationMatrix(const double m[3][3])
 		// Computation depends on major diagonal term
 		if ((m[0][0] > m[1][1])&(m[0][0] > m[2][2]))
 		{
-			const double s = sqrt(1.0 + m[0][0] - m[1][1] - m[2][2]) * 2.0;
+			const qreal s = sqrt(1.0 + m[0][0] - m[1][1] - m[2][2]) * 2.0;
 			q[0] = 0.25 * s;
 			q[1] = (m[0][1] + m[1][0]) / s;
 			q[2] = (m[0][2] + m[2][0]) / s;
@@ -129,7 +129,7 @@ void Quaternion::setFromRotationMatrix(const double m[3][3])
 		else
 			if (m[1][1] > m[2][2])
 			{
-				const double s = sqrt(1.0 + m[1][1] - m[0][0] - m[2][2]) * 2.0;
+				const qreal s = sqrt(1.0 + m[1][1] - m[0][0] - m[2][2]) * 2.0;
 				q[0] = (m[0][1] + m[1][0]) / s;
 				q[1] = 0.25 * s;
 				q[2] = (m[1][2] + m[2][1]) / s;
@@ -137,7 +137,7 @@ void Quaternion::setFromRotationMatrix(const double m[3][3])
 			}
 			else
 			{
-				const double s = sqrt(1.0 + m[2][2] - m[0][0] - m[1][1]) * 2.0;
+				const qreal s = sqrt(1.0 + m[2][2] - m[0][0] - m[1][1]) * 2.0;
 				q[0] = (m[0][2] + m[2][0]) / s;
 				q[1] = (m[1][2] + m[2][1]) / s;
 				q[2] = 0.25 * s;
@@ -150,12 +150,12 @@ void Quaternion::setFromRotationMatrix(const double m[3][3])
 #ifndef DOXYGEN
 void Quaternion::setFromRotationMatrix(const float m[3][3])
 {
-	qWarning("setFromRotationMatrix now waits for a double[3][3] parameter");
+	qWarning("setFromRotationMatrix now expects a double[3][3] parameter");
 
-	double mat[3][3];
+	qreal mat[3][3];
 	for (int i=0; i<3; ++i)
 		for (int j=0; j<3; ++j)
-			mat[i][j] = double(m[i][j]);
+			mat[i][j] = qreal(m[i][j]);
 
 	setFromRotationMatrix(mat);
 }
@@ -181,10 +181,10 @@ void Quaternion::setFromRotatedBase(const Vec& X, const Vec& Y, const Vec& Z)
   See also setFromRotationMatrix() and Quaternion(const Vec&, const Vec&). */
 void Quaternion::setFromRotatedBasis(const Vec& X, const Vec& Y, const Vec& Z)
 {
-	double m[3][3];
-	double normX = X.norm();
-	double normY = Y.norm();
-	double normZ = Z.norm();
+	qreal m[3][3];
+	qreal normX = X.norm();
+	qreal normY = Y.norm();
+	qreal normZ = Z.norm();
 
 	for (int i=0; i<3; ++i)
 	{
@@ -198,17 +198,17 @@ void Quaternion::setFromRotatedBasis(const Vec& X, const Vec& Y, const Vec& Z)
 
 /*! Returns the axis vector and the angle (in radians) of the rotation represented by the Quaternion.
  See the axis() and angle() documentations. */
-void Quaternion::getAxisAngle(Vec& axis, float& angle) const
+void Quaternion::getAxisAngle(Vec& axis, qreal& angle) const
 {
-	angle = 2.0*acos(q[3]);
+	angle = 2.0 * acos(q[3]);
 	axis = Vec(q[0], q[1], q[2]);
-	const double sinus = axis.norm();
+	const qreal sinus = axis.norm();
 	if (sinus > 1E-8)
 		axis /= sinus;
 
 	if (angle > M_PI)
 	{
-		angle = 2.0*M_PI - angle;
+		angle = 2.0 * qreal(M_PI) - angle;
 		axis = -axis;
 	}
 }
@@ -219,7 +219,7 @@ It is null for an identity Quaternion. See also angle() and getAxisAngle(). */
 Vec Quaternion::axis() const
 {
 	Vec res = Vec(q[0], q[1], q[2]);
-	const double sinus = res.norm();
+	const qreal sinus = res.norm();
 	if (sinus > 1E-8)
 		res /= sinus;
 	return (acos(q[3]) <= M_PI/2.0) ? res : -res;
@@ -231,9 +231,9 @@ Vec Quaternion::axis() const
  axis() direction.
 
  See also axis() and getAxisAngle(). */
-double Quaternion::angle() const
+qreal Quaternion::angle() const
 {
-	const double angle = 2.0 * acos(q[3]);
+	const qreal angle = 2.0 * acos(q[3]);
 	return (angle <= M_PI) ? angle : 2.0*M_PI - angle;
 }
 
@@ -287,12 +287,8 @@ Quaternion::Quaternion(const QDomElement& element)
 {
 	QStringList attribute;
 	attribute << "q0" << "q1" << "q2" << "q3";
-#if QT_VERSION >= 0x040000
 	for (int i=0; i<attribute.size(); ++i)
-#else
-	for (unsigned int i=0; i<attribute.count(); ++i)
-#endif
-		q[i] = DomUtils::doubleFromDom(element, attribute[i], ((i<3)?0.0f:1.0f));
+		q[i] = DomUtils::qrealFromDom(element, attribute[i], ((i<3)?0.0:1.0));
 }
 
 /*! Returns the Quaternion associated 4x4 OpenGL rotation matrix.
@@ -320,39 +316,39 @@ Use matrix() if you do not need to store this matrix and simply want to alter th
 matrix. See also getInverseMatrix() and Frame::getMatrix(). */
 void Quaternion::getMatrix(GLdouble m[4][4]) const
 {
-	const double q00 = 2.0l * q[0] * q[0];
-	const double q11 = 2.0l * q[1] * q[1];
-	const double q22 = 2.0l * q[2] * q[2];
+	const qreal q00 = 2.0 * q[0] * q[0];
+	const qreal q11 = 2.0 * q[1] * q[1];
+	const qreal q22 = 2.0 * q[2] * q[2];
 
-	const double q01 = 2.0l * q[0] * q[1];
-	const double q02 = 2.0l * q[0] * q[2];
-	const double q03 = 2.0l * q[0] * q[3];
+	const qreal q01 = 2.0 * q[0] * q[1];
+	const qreal q02 = 2.0 * q[0] * q[2];
+	const qreal q03 = 2.0 * q[0] * q[3];
 
-	const double q12 = 2.0l * q[1] * q[2];
-	const double q13 = 2.0l * q[1] * q[3];
+	const qreal q12 = 2.0 * q[1] * q[2];
+	const qreal q13 = 2.0 * q[1] * q[3];
 
-	const double q23 = 2.0l * q[2] * q[3];
+	const qreal q23 = 2.0 * q[2] * q[3];
 
-	m[0][0] = 1.0l - q11 - q22;
-	m[1][0] =        q01 - q23;
-	m[2][0] =        q02 + q13;
+	m[0][0] = 1.0 - q11 - q22;
+	m[1][0] =       q01 - q23;
+	m[2][0] =       q02 + q13;
 
-	m[0][1] =        q01 + q23;
-	m[1][1] = 1.0l - q22 - q00;
-	m[2][1] =        q12 - q03;
+	m[0][1] =       q01 + q23;
+	m[1][1] = 1.0 - q22 - q00;
+	m[2][1] =       q12 - q03;
 
-	m[0][2] =        q02 - q13;
-	m[1][2] =        q12 + q03;
-	m[2][2] = 1.0l - q11 - q00;
+	m[0][2] =       q02 - q13;
+	m[1][2] =       q12 + q03;
+	m[2][2] = 1.0 - q11 - q00;
 
-	m[0][3] = 0.0l;
-	m[1][3] = 0.0l;
-	m[2][3] = 0.0l;
+	m[0][3] = 0.0;
+	m[1][3] = 0.0;
+	m[2][3] = 0.0;
 
-	m[3][0] = 0.0l;
-	m[3][1] = 0.0l;
-	m[3][2] = 0.0l;
-	m[3][3] = 1.0l;
+	m[3][0] = 0.0;
+	m[3][1] = 0.0;
+	m[3][2] = 0.0;
+	m[3][3] = 1.0;
 }
 
 /*! Same as getMatrix(), but with a \c GLdouble[16] parameter. See also getInverseMatrix() and Frame::getMatrix(). */
@@ -372,14 +368,14 @@ void Quaternion::getMatrix(GLdouble m[16]) const
 
   \attention \p m uses the European mathematical representation of the rotation matrix. Use matrix()
   and getMatrix() to retrieve the OpenGL transposed version. */
-void Quaternion::getRotationMatrix(float m[3][3]) const
+void Quaternion::getRotationMatrix(qreal m[3][3]) const
 {
 	static GLdouble mat[4][4];
 	getMatrix(mat);
 	for (int i=0; i<3; ++i)
 		for (int j=0; j<3; ++j)
 			// Beware of transposition
-			m[i][j] = mat[j][i];
+			m[i][j] = qreal(mat[j][i]);
 }
 
 /*! Returns the associated 4x4 OpenGL \e inverse rotation matrix. This is simply the matrix() of the
@@ -416,14 +412,14 @@ void Quaternion::getInverseMatrix(GLdouble m[16]) const
 
  \attention This is the classical mathematical rotation matrix. The OpenGL format uses its
  transposed version. See inverseMatrix() and getInverseMatrix(). */
-void Quaternion::getInverseRotationMatrix(float m[3][3]) const
+void Quaternion::getInverseRotationMatrix(qreal m[3][3]) const
 {
 	static GLdouble mat[4][4];
 	getInverseMatrix(mat);
 	for (int i=0; i<3; ++i)
 		for (int j=0; j<3; ++j)
 			// Beware of transposition
-			m[i][j] = mat[j][i];
+			m[i][j] = qreal(mat[j][i]);
 }
 
 
@@ -434,11 +430,11 @@ void Quaternion::getInverseRotationMatrix(float m[3][3]) const
  When \p allowFlip is \c true (default) the slerp interpolation will always use the "shortest path"
  between the Quaternions' orientations, by "flipping" the source Quaternion if needed (see
  negate()). */
-Quaternion Quaternion::slerp(const Quaternion& a, const Quaternion& b, float t, bool allowFlip)
+Quaternion Quaternion::slerp(const Quaternion& a, const Quaternion& b, qreal t, bool allowFlip)
 {
-	double cosAngle = Quaternion::dot(a, b);
+	qreal cosAngle = Quaternion::dot(a, b);
 
-	double c1, c2;
+	qreal c1, c2;
 	// Linear interpolation for close orientations
 	if ((1.0 - fabs(cosAngle)) < 0.01)
 	{
@@ -448,8 +444,8 @@ Quaternion Quaternion::slerp(const Quaternion& a, const Quaternion& b, float t,
 	else
 	{
 		// Spherical interpolation
-		double angle    = acos(fabs(cosAngle));
-		double sinAngle = sin(angle);
+		qreal angle    = acos(fabs(cosAngle));
+		qreal sinAngle = sin(angle);
 		c1 = sin(angle * (1.0 - t)) / sinAngle;
 		c2 = sin(angle * t) / sinAngle;
 	}
@@ -468,7 +464,7 @@ Quaternion Quaternion::slerp(const Quaternion& a, const Quaternion& b, float t,
   t=1).
 
   Use squadTangent() to define the Quaternion tangents \p tgA and \p tgB. */
-Quaternion Quaternion::squad(const Quaternion& a, const Quaternion& tgA, const Quaternion& tgB, const Quaternion& b, float t)
+Quaternion Quaternion::squad(const Quaternion& a, const Quaternion& tgA, const Quaternion& tgB, const Quaternion& b, qreal t)
 {
 	Quaternion ab = Quaternion::slerp(a, b, t);
 	Quaternion tg = Quaternion::slerp(tgA, tgB, t, false);
@@ -478,13 +474,13 @@ Quaternion Quaternion::squad(const Quaternion& a, const Quaternion& tgA, const Q
 /*! Returns the logarithm of the Quaternion. See also exp(). */
 Quaternion Quaternion::log()
 {
-	double len = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
+	qreal len = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
 
 	if (len < 1E-6)
 		return Quaternion(q[0], q[1], q[2], 0.0);
 	else
 	{
-		double coef = acos(q[3]) / len;
+		qreal coef = acos(q[3]) / len;
 		return Quaternion(q[0]*coef, q[1]*coef, q[2]*coef, 0.0);
 	}
 }
@@ -492,13 +488,13 @@ Quaternion Quaternion::log()
 /*! Returns the exponential of the Quaternion. See also log(). */
 Quaternion Quaternion::exp()
 {
-	double theta = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
+	qreal theta = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
 
 	if (theta < 1E-6)
 		return Quaternion(q[0], q[1], q[2], cos(theta));
 	else
 	{
-		double coef = sin(theta) / theta;
+		qreal coef = sin(theta) / theta;
 		return Quaternion(q[0]*coef, q[1]*coef, q[2]*coef, cos(theta));
 	}
 }
@@ -547,10 +543,10 @@ Quaternion Quaternion::randomQuaternion()
 {
 	// The rand() function is not very portable and may not be available on your system.
 	// Add the appropriate include or replace by an other random function in case of problem.
-	double seed = rand()/(double)RAND_MAX;
-	double r1 = sqrt(1.0 - seed);
-	double r2 = sqrt(seed);
-	double t1 = 2.0 * M_PI * (rand()/(double)RAND_MAX);
-	double t2 = 2.0 * M_PI * (rand()/(double)RAND_MAX);
+	qreal seed = rand()/(qreal)RAND_MAX;
+	qreal r1 = sqrt(1.0 - seed);
+	qreal r2 = sqrt(seed);
+	qreal t1 = 2.0 * M_PI * (rand()/(qreal)RAND_MAX);
+	qreal t2 = 2.0 * M_PI * (rand()/(qreal)RAND_MAX);
 	return Quaternion(sin(t1)*r1, cos(t1)*r1, sin(t2)*r2, cos(t2)*r2);
 }
