@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -23,30 +23,14 @@
 #include "qglviewer.h"
 
 #ifndef NO_VECTORIAL_RENDER
-# if QT_VERSION >= 0x040000
 #  include "ui_VRenderInterface.h"
-# else
-#  include <qcheckbox.h>
-#  include <qcombobox.h>
-#  include "VRenderInterface.Qt3.h"
-# endif
-# include "VRender/VRender.h"
+#  include "VRender/VRender.h"
 #endif
 
-#if QT_VERSION >= 0x040000
-# include "ui_ImageInterface.h"
-#else
-# include <qspinbox.h>
-# include <qcheckbox.h>
-# include "ImageInterface.Qt3.h"
-#endif
+#include "ui_ImageInterface.h"
 
 // Output format list
-#if QT_VERSION < 0x040000
-# include <qimage.h>
-#else
 # include <QImageWriter>
-#endif
 
 #include <qfileinfo.h>
 #include <qfiledialog.h>
@@ -73,11 +57,7 @@ static QMap<QString, QString> extension;
 /*! Sets snapshotFileName(). */
 void QGLViewer::setSnapshotFileName(const QString& name)
 {
-#if QT_VERSION >= 0x040000
 	snapshotFileName_ = QFileInfo(name).absoluteFilePath();
-#else
-	snapshotFileName_ = QFileInfo(name).absFilePath();
-#endif
 }
 
 #ifndef DOXYGEN
@@ -97,15 +77,9 @@ Returns \c false if the user presses the Cancel button and \c true otherwise. */
 bool QGLViewer::openSnapshotFormatDialog()
 {
 	bool ok = false;
-#if QT_VERSION >= 0x040000
 	QStringList list = formats.split(";;", QString::SkipEmptyParts);
 	int current = list.indexOf(FDFormatString[snapshotFormat()]);
 	QString format = QInputDialog::getItem(this, "Snapshot format", "Select a snapshot format", list, current, false, &ok);
-#else
-	QStringList list = QStringList::split(";;", formats);
-	int current = list.findIndex(FDFormatString[snapshotFormat()]);
-	QString format = QInputDialog::getItem("Snapshot format", "Select a snapshot format", list, current, false, &ok, this);
-#endif
 	if (ok)
 		setSnapshotFormat(Qtformat[format]);
 	return ok;
@@ -116,14 +90,10 @@ bool QGLViewer::openSnapshotFormatDialog()
 // saveSnapshot dialog. Initialize snapshotFormat() to the first one.
 void QGLViewer::initializeSnapshotFormats()
 {
-#if QT_VERSION >= 0x040000
 	QList<QByteArray> list = QImageWriter::supportedImageFormats();
 	QStringList formatList;
 	for (int i=0; i < list.size(); ++i)
 		formatList << QString(list.at(i).toUpper());
-#else
-	QStringList formatList = QImage::outputFormatList();
-#endif
 	//        qWarning("Available image formats: ");
 	//        QStringList::Iterator it = formatList.begin();
 	//        while( it != formatList.end() )
@@ -147,15 +117,10 @@ void QGLViewer::initializeSnapshotFormats()
 	QtText += "BMP";	MenuText += "Windows Bitmap (*.bmp)";	Ext += "bmp";
 	QtText += "XFIG";	MenuText += "XFig (*.fig)";		Ext += "fig";
 
-#if QT_VERSION < 0x030000
-	QStringList::Iterator itText = QtText.begin();
-	QStringList::Iterator itMenu = MenuText.begin();
-	QStringList::Iterator itExt  = Ext.begin();
-#else
 	QStringList::iterator itText = QtText.begin();
 	QStringList::iterator itMenu = MenuText.begin();
 	QStringList::iterator itExt  = Ext.begin();
-#endif
+
 	while (itText != QtText.end())
 	{
 		//QMessageBox::information(this, "Snapshot ", "Trying format\n"+(*itText));
@@ -187,11 +152,7 @@ static bool checkFileName(QString& fileName, QWidget* widget, const QString& sna
 	// Check that extension has been provided
 	QFileInfo info(fileName);
 
-#if QT_VERSION >= 0x040000
 	if (info.suffix().isEmpty())
-#else
-	if (info.extension(false).isEmpty())
-#endif
 	{
 		// No extension given. Silently add one
 		if (fileName.right(1) != ".")
@@ -199,22 +160,10 @@ static bool checkFileName(QString& fileName, QWidget* widget, const QString& sna
 		fileName += extension[snapshotFormat];
 		info.setFile(fileName);
 	}
-#if QT_VERSION >= 0x040000
 	else if (info.suffix() != extension[snapshotFormat])
-#else
-	else if (info.extension(false) != extension[snapshotFormat])
-#endif
 	{
 		// Extension is not appropriate. Propose a modification
-#if QT_VERSION >= 0x040000
 		QString modifiedName = info.absolutePath() + '/' + info.baseName() + "." + extension[snapshotFormat];
-#else
-# if QT_VERSION >= 0x030000
-		QString modifiedName = info.dirPath() + '/' + info.baseName(true) + '.' + extension[snapshotFormat];
-# else
-		QString modifiedName = info.dirPath() + '/' + info.baseName() + '.' + extension[snapshotFormat];
-# endif
-#endif
 		QFileInfo modifInfo(modifiedName);
 		int i=(QMessageBox::warning(widget,"Wrong extension",
 									info.fileName()+" has a wrong extension.\nSave as "+modifInfo.fileName()+" instead ?",
@@ -258,11 +207,7 @@ QProgressDialog* ProgressDialog::progressDialog = NULL;
 void ProgressDialog::showProgressDialog(QGLWidget* parent)
 {
 	progressDialog = new QProgressDialog(parent);
-#if QT_VERSION >= 0x040000
 	progressDialog->setWindowTitle("Image rendering progress");
-#else
-	progressDialog->setCaption("Image rendering progress");
-#endif
 	progressDialog->setMinimumSize(300, 40);
 	progressDialog->setCancelButton(NULL);
 	progressDialog->show();
@@ -270,11 +215,7 @@ void ProgressDialog::showProgressDialog(QGLWidget* parent)
 
 void ProgressDialog::updateProgress(float progress, const QString& stepString)
 {
-#if QT_VERSION >= 0x040000
 	progressDialog->setValue(int(progress*100));
-#else
-	progressDialog->setProgress(int(progress*100));
-#endif
 	QString message(stepString);
 	if (message.length() > 33)
 		message = message.left(17) + "..." + message.right(12);
@@ -290,14 +231,12 @@ void ProgressDialog::hideProgressDialog()
 	progressDialog = NULL;
 }
 
-#if QT_VERSION >= 0x040000
 class VRenderInterface: public QDialog, public Ui::VRenderInterface
 {
 public: VRenderInterface(QWidget *parent) : QDialog(parent) { setupUi(this); }
 };
-#endif
 
-#endif
+#endif //DOXYGEN
 
 // Pops-up a vectorial output option dialog box and save to fileName
 // Returns -1 in case of Cancel, 0 for success and (todo) error code in case of problem.
@@ -306,11 +245,7 @@ static int saveVectorialSnapshot(const QString& fileName, QGLWidget* widget, con
 	static VRenderInterface* VRinterface = NULL;
 
 	if (!VRinterface)
-#if QT_VERSION >= 0x030000
 		VRinterface = new VRenderInterface(widget);
-#else
-		VRinterface = new VRenderInterface(widget, "", true); // Make the dialog modal
-#endif
 
 
 	// Configure interface according to selected snapshotFormat
@@ -341,18 +276,14 @@ static int saveVectorialSnapshot(const QString& fileName, QGLWidget* widget, con
 	vparams.setOption(vrender::VRenderParams::AddBackground, VRinterface->colorBackground->isChecked());
 	vparams.setOption(vrender::VRenderParams::TightenBoundingBox, VRinterface->tightenBBox->isChecked());
 
-#if QT_VERSION >= 0x040000
 	switch (VRinterface->sortMethod->currentIndex())
-#else
-	switch (VRinterface->sortMethod->currentItem())
-#endif
 	{
-	case 0: vparams.setSortMethod(vrender::VRenderParams::NoSorting); 		break;
-	case 1: vparams.setSortMethod(vrender::VRenderParams::BSPSort); 		break;
-	case 2: vparams.setSortMethod(vrender::VRenderParams::TopologicalSort); 	break;
-	case 3: vparams.setSortMethod(vrender::VRenderParams::AdvancedTopologicalSort);	break;
-	default:
-		qWarning("VRenderInterface::saveVectorialSnapshot: Unknown SortMethod");
+		case 0: vparams.setSortMethod(vrender::VRenderParams::NoSorting); 		break;
+		case 1: vparams.setSortMethod(vrender::VRenderParams::BSPSort); 		break;
+		case 2: vparams.setSortMethod(vrender::VRenderParams::TopologicalSort); 	break;
+		case 3: vparams.setSortMethod(vrender::VRenderParams::AdvancedTopologicalSort);	break;
+		default:
+			qWarning("VRenderInterface::saveVectorialSnapshot: Unknown SortMethod");
 	}
 
 	vparams.setProgressFunction(&ProgressDialog::updateProgress);
@@ -361,11 +292,7 @@ static int saveVectorialSnapshot(const QString& fileName, QGLWidget* widget, con
 	widget->raise();
 	vrender::VectorialRender(drawVectorial, (void*) widget, vparams);
 	ProgressDialog::hideProgressDialog();
-#if QT_VERSION < 0x030000
-	widget->setCursor(Qt::arrowCursor);
-#else
 	widget->setCursor(QCursor(Qt::ArrowCursor));
-#endif
 
 	// Should return vparams.error(), but this is currently not set.
 	return 0;
@@ -373,12 +300,10 @@ static int saveVectorialSnapshot(const QString& fileName, QGLWidget* widget, con
 #endif // NO_VECTORIAL_RENDER
 
 
-#if QT_VERSION >= 0x040000
 class ImageInterface: public QDialog, public Ui::ImageInterface
 {
 public: ImageInterface(QWidget *parent) : QDialog(parent) { setupUi(this); }
 };
-#endif
 
 
 // Pops-up an image settings dialog box and save to fileName.
@@ -388,11 +313,7 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 	static ImageInterface* imageInterface = NULL;
 
 	if (!imageInterface)
-#if QT_VERSION >= 0x030000
 		imageInterface = new ImageInterface(this);
-#else
-		imageInterface = new ImageInterface(this, "", true);  // Make the dialog modal
-#endif
 
 	imageInterface->imgWidth->setValue(width());
 	imageInterface->imgHeight->setValue(height());
@@ -413,16 +334,16 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 
 	QSize finalSize(imageInterface->imgWidth->value(), imageInterface->imgHeight->value());
 
-	double oversampling = imageInterface->oversampling->value();
+	qreal oversampling = imageInterface->oversampling->value();
 	QSize subSize(int(this->width()/oversampling), int(this->height()/oversampling));
 
-	double aspectRatio = width() / static_cast<double>(height());
-	double newAspectRatio = finalSize.width() / static_cast<double>(finalSize.height());
+	qreal aspectRatio = width() / static_cast<qreal>(height());
+	qreal newAspectRatio = finalSize.width() / static_cast<qreal>(finalSize.height());
 
-	double zNear = camera()->zNear();
-	double zFar = camera()->zFar();
+	qreal zNear = camera()->zNear();
+	qreal zFar = camera()->zFar();
 
-	double xMin, yMin;
+	qreal xMin, yMin;
 	bool expand = imageInterface->expandFrustum->isChecked();
 	if (camera()->type() == qglviewer::Camera::PERSPECTIVE)
 		if ((expand && (newAspectRatio>aspectRatio)) || (!expand && (newAspectRatio<aspectRatio)))
@@ -444,11 +365,7 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 			yMin = xMin / newAspectRatio;
 	}
 
-#if QT_VERSION >= 0x040000
 	QImage image(finalSize.width(), finalSize.height(), QImage::Format_ARGB32);
-#else
-	QImage image(finalSize.width(), finalSize.height(), 32);
-#endif
 
 	if (image.isNull())
 	{
@@ -461,11 +378,11 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 	// ProgressDialog disabled since it interfers with the screen grabing mecanism on some platforms. Too bad.
 	// ProgressDialog::showProgressDialog(this);
 
-	double scaleX = subSize.width() / static_cast<double>(finalSize.width());
-	double scaleY = subSize.height() / static_cast<double>(finalSize.height());
+	qreal scaleX = subSize.width() / static_cast<qreal>(finalSize.width());
+	qreal scaleY = subSize.height() / static_cast<qreal>(finalSize.height());
 
-	double deltaX = 2.0 * xMin * scaleX;
-	double deltaY = 2.0 * yMin * scaleY;
+	qreal deltaX = 2.0 * xMin * scaleX;
+	qreal deltaY = 2.0 * yMin * scaleY;
 
 	int nbX = finalSize.width() / subSize.width();
 	int nbY = finalSize.height() / subSize.height();
@@ -481,10 +398,10 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 	// tileRegion_ is used by startScreenCoordinatesSystem to appropriately set the local
 	// coordinate system when tiling
 	tileRegion_ = new TileRegion();
-	double tileXMin, tileWidth, tileYMin, tileHeight;
+	qreal tileXMin, tileWidth, tileYMin, tileHeight;
 	if ((expand && (newAspectRatio>aspectRatio)) || (!expand && (newAspectRatio<aspectRatio)))
 	{
-		double tileTotalWidth = newAspectRatio * height();
+		qreal tileTotalWidth = newAspectRatio * height();
 		tileXMin = (width() - tileTotalWidth) / 2.0;
 		tileWidth = tileTotalWidth * scaleX;
 		tileYMin = 0.0;
@@ -493,7 +410,7 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 	}
 	else
 	{
-		double tileTotalHeight = width() / newAspectRatio;
+		qreal tileTotalHeight = width() / newAspectRatio;
 		tileYMin = (height() - tileTotalHeight) / 2.0;
 		tileHeight = tileTotalHeight * scaleY;
 		tileXMin = 0.0;
@@ -530,19 +447,11 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 			QImage snapshot = grabFrameBuffer(true);
 
 			// ProgressDialog::showProgressDialog(this);
-			// ProgressDialog::updateProgress(count / (float)(nbX*nbY),
+			// ProgressDialog::updateProgress(count / (qreal)(nbX*nbY),
 			// "Generating image ["+QString::number(count)+"/"+QString::number(nbX*nbY)+"]");
 			// qApp->processEvents();
 
-#if QT_VERSION >= 0x040000
 			QImage subImage = snapshot.scaled(subSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-#else
-# if QT_VERSION >= 0x030000
-			QImage subImage = snapshot.scale(subSize, QImage::ScaleFree);
-# else
-			QImage subImage = snapshot.smoothScale(subSize.width(), subSize.height());
-# endif
-#endif
 
 			// Copy subImage in image
 			for (int ii=0; ii<subSize.width(); ii++)
@@ -561,19 +470,10 @@ bool QGLViewer::saveImageSnapshot(const QString& fileName)
 			count++;
 		}
 
-#if QT_VERSION >= 0x040000
 	bool saveOK = image.save(fileName, snapshotFormat().toLatin1().constData(), snapshotQuality());
-#else
-	bool saveOK = image.save(fileName, snapshotFormat(), snapshotQuality());
-#endif
 
 	// ProgressDialog::hideProgressDialog();
-
-	// #if QT_VERSION < 0x030000
-	// setCursor(Qt::arrowCursor);
-	// #else
 	// setCursor(QCursor(Qt::ArrowCursor));
-	// #endif
 
 	delete tileRegion_;
 	tileRegion_ = NULL;
@@ -640,22 +540,10 @@ void QGLViewer::saveSnapshot(bool automatic, bool overwrite)
 	if (snapshotFileName().isEmpty() || !automatic)
 	{
 		QString fileName;
-#if QT_VERSION < 0x030000
-		if (openSnapshotFormatDialog())
-			fileName = QFileDialog::getSaveFileName(snapshotFileName(), FDFormatString[snapshotFormat()]+";;All files (*.*)", this, "Save dialog");
-		else
-			return;
-#else
 		QString selectedFormat = FDFormatString[snapshotFormat()];
-# if QT_VERSION >= 0x040000
 		fileName = QFileDialog::getSaveFileName(this, "Choose a file name to save under", snapshotFileName(), formats, &selectedFormat,
-												overwrite?QFileDialog::DontConfirmOverwrite:QFlag(0));
-# else
-		fileName = QFileDialog::getSaveFileName(snapshotFileName(), formats, this,
-												"Save Snapshot dialog", "Choose a file name to save under", &selectedFormat);
-# endif
+												overwrite?QFileDialog::DontConfirmOverwrite:QFlags<QFileDialog::Option>(0));
 		setSnapshotFormat(Qtformat[selectedFormat]);
-#endif
 
 		if (checkFileName(fileName, this, snapshotFormat()))
 			setSnapshotFileName(fileName);
@@ -672,39 +560,19 @@ void QGLViewer::saveSnapshot(bool automatic, bool overwrite)
 		QString count;
 		count.sprintf("%.04d", snapshotCounter_++);
 		QString suffix;
-#if QT_VERSION >= 0x040000
 		suffix = fileInfo.suffix();
 		if (suffix.isEmpty())
 			suffix = extension[snapshotFormat()];
 		fileInfo.setFile(fileInfo.absolutePath()+ '/' + baseName + '-' + count + '.' + suffix);
-#else      
-		suffix = fileInfo.extension();
-		if (suffix.isEmpty())
-			suffix = extension[snapshotFormat()];
-		fileInfo.setFile(fileInfo.dirPath()+ '/' + baseName + '-' + count + '.' + suffix);
-#endif
 
 		if (!overwrite)
 			while (fileInfo.exists())
 			{
 				count.sprintf("%.04d", snapshotCounter_++);
-#if QT_VERSION >= 0x040000
 				fileInfo.setFile(fileInfo.absolutePath() + '/' +baseName + '-' + count + '.' + fileInfo.suffix());
-#else
-				fileInfo.setFile(fileInfo.dirPath() + '/' + baseName + '-' + count + '.' + fileInfo.extension());
-#endif
 			}
 	}
 
-#if QT_VERSION < 0x040000
-	if ((fileInfo.exists()) && (!overwrite) &&
-			(QMessageBox::warning(this,"Overwrite file ?",
-								  "File "+fileInfo.fileName()+" already exists.\nOverwrite ?",
-								  QMessageBox::Yes,
-								  QMessageBox::Cancel) == QMessageBox::Cancel))
-		return;
-#endif
-
 	bool saveOK;
 #ifndef NO_VECTORIAL_RENDER
 	if ( (snapshotFormat() == "EPS") || (snapshotFormat() == "PS") || (snapshotFormat() == "XFIG") )
@@ -715,11 +583,7 @@ void QGLViewer::saveSnapshot(bool automatic, bool overwrite)
 		if (automatic)
 		{
 			QImage snapshot = frameBufferSnapshot();
-#if QT_VERSION >= 0x040000
 			saveOK = snapshot.save(fileInfo.filePath(), snapshotFormat().toLatin1().constData(), snapshotQuality());
-#else
-			saveOK = snapshot.save(fileInfo.filePath(), snapshotFormat(), snapshotQuality());
-#endif
 		}
 		else
 			saveOK = saveImageSnapshot(fileInfo.filePath());
@@ -772,63 +636,3 @@ void QGLViewer::snapshotToClipboard()
 	cb->setImage(frameBufferSnapshot());
 }
 
-#if QT_VERSION < 0x030000
-// This code is largely inspired from Qt's method available in version 3
-// Copyright Trolltech AS
-QImage QGLViewer::grabFrameBuffer(bool withAlpha)
-{
-	makeCurrent();
-	QImage res;
-	int w = width();
-	int h = height();
-	if (format().rgba())
-	{
-		res = QImage(w, h, 32);
-		glReadPixels(0, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, res.bits());
-		if (QImage::systemByteOrder() == QImage::BigEndian)
-		{
-			// OpenGL gives RGBA; Qt wants ARGB
-			uint *p = (uint*)res.bits();
-			uint *end = p + w*h;
-			if (withAlpha && format().alpha())
-			{
-				while (p < end)
-				{
-					uint a = *p << 24;
-					*p = (*p >> 8) | a;
-					p++;
-				}
-			}
-			else
-			{
-				while (p < end)
-					*p++ >>= 8;
-			}
-		}
-		else
-		{
-			// OpenGL gives ABGR (i.e. RGBA backwards); Qt wants ARGB
-			res = res.swapRGB();
-		}
-		res.setAlphaBuffer(withAlpha && format().alpha());
-	}
-	else
-	{
-#if defined (Q_OS_WIN) || defined (Q_WS_WIN)
-		res = QImage(w, h, 8);
-		glReadPixels(0, 0, w, h, GL_COLOR_INDEX, GL_UNSIGNED_BYTE, res.bits());
-		//int palSize = 0;
-		//const QRgb* pal = QColor::palette(&palSize);
-		//if (pal && palSize)
-		//{
-		//  res.setNumColors(palSize);
-		//  for (int i = 0; i < palSize; i++)
-		//	res.setColor(i, pal[i]);
-		//}
-#endif
-	}
-
-	return res.mirror();
-}
-
-#endif // QT_VERSION < 0x030000
