@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -25,11 +25,7 @@
 class Viewer : public QGLViewer
 {
 public :
-#if QT_VERSION < 0x040000
-  Viewer(QWidget *parent, const char *name);
-#else
   Viewer(QWidget *parent);
-#endif
 
 protected :
   virtual void draw();
