@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -23,24 +23,15 @@
 #include "QGLViewer/constraint.h"
 #include "object.h"
 
-#if QT_VERSION < 0x040000
-# include <qptrlist.h> 
-#endif
-
 class ManipulatedFrameSetConstraint : public qglviewer::Constraint
 {
 public:
-  void clearSet();
-  void addObjectToSet(Object* o);
+	void clearSet();
+	void addObjectToSet(Object* o);
 
-  virtual void constrainTranslation(qglviewer::Vec &translation, qglviewer::Frame *const frame);
-  virtual void constrainRotation(qglviewer::Quaternion &rotation, qglviewer::Frame *const frame);
+	virtual void constrainTranslation(qglviewer::Vec &translation, qglviewer::Frame *const frame);
+	virtual void constrainRotation(qglviewer::Quaternion &rotation, qglviewer::Frame *const frame);
 
 private :
-#if QT_VERSION < 0x040000
-  // Objects of the scene
-  QPtrList<Object> objects_;
-#else
-  QList<Object*> objects_;
-#endif
+	QList<Object*> objects_;
 };
