@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -28,11 +28,7 @@ int main(int argc, char** argv)
   QApplication application(argc,argv);
   Viewer viewer;
 
-#if QT_VERSION < 0x040000
-  application.setMainWidget(&viewer);
-#else
   viewer.setWindowTitle("overpainting");
-#endif
 
   viewer.show();
   return application.exec();
