@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.5.0.
+ This file is part of the QGLViewer library version 2.6.3.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -27,8 +27,8 @@ class StandardCamera : public qglviewer::Camera
 public :
   StandardCamera();
 
-  virtual float zNear() const;
-  virtual float zFar() const;
+  virtual qreal zNear() const;
+  virtual qreal zFar() const;
 
   void toggleMode() { standard = !standard; }
   bool isStandard() { return standard; }
